@@ -83,6 +83,7 @@ gdk_pixbuf_finalize (GObject *object)
         
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
+
 
 /**
  * gdk_pixbuf_ref:
@@ -95,7 +96,7 @@ gdk_pixbuf_finalize (GObject *object)
 GdkPixbuf *
 gdk_pixbuf_ref (GdkPixbuf *pixbuf)
 {
-        return (GdkPixbuf *) g_object_ref (G_OBJECT(pixbuf));
+        return (GdkPixbuf *) g_object_ref (pixbuf);
 }
 
 /**
@@ -109,7 +110,7 @@ gdk_pixbuf_ref (GdkPixbuf *pixbuf)
 void
 gdk_pixbuf_unref (GdkPixbuf *pixbuf)
 {
-        g_object_unref (G_OBJECT (pixbuf));
+        g_object_unref (pixbuf);
 }
 
 
@@ -129,32 +130,46 @@ free_buffer (guchar *pixels, gpointer data)
  * @width: Width of image in pixels.
  * @height: Height of image in pixels.
  *
- * Creates a new #GdkPixbuf structure and allocates a buffer for it.  The buffer
- * has an optimal rowstride.  Note that the buffer is not cleared; you will have
- * to fill it completely yourself.
+ * Creates a new #GdkPixbuf structure and allocates a buffer for it.  The 
+ * buffer has an optimal rowstride.  Note that the buffer is not cleared;
+ * you will have to fill it completely yourself.
  *
- * Return value: A newly-created #GdkPixbuf with a reference count of 1, or NULL
- * if not enough memory could be allocated for the image buffer.
+ * Return value: A newly-created #GdkPixbuf with a reference count of 1, or 
+ * %NULL if not enough memory could be allocated for the image buffer.
  **/
 GdkPixbuf *
-gdk_pixbuf_new (GdkColorspace colorspace, gboolean has_alpha, int bits_per_sample,
-		int width, int height)
+gdk_pixbuf_new (GdkColorspace colorspace, 
+                gboolean      has_alpha,
+                int           bits_per_sample,
+                int           width,
+                int           height)
 {
 	guchar *buf;
 	int channels;
 	int rowstride;
+        gsize bytes;
 
 	g_return_val_if_fail (colorspace == GDK_COLORSPACE_RGB, NULL);
 	g_return_val_if_fail (bits_per_sample == 8, NULL);
 	g_return_val_if_fail (width > 0, NULL);
 	g_return_val_if_fail (height > 0, NULL);
 
-	/* Always align rows to 32-bit boundaries */
+        if (width <= 0 || height <= 0)
+                return NULL;
 
 	channels = has_alpha ? 4 : 3;
-	rowstride = 4 * ((channels * width + 3) / 4);
+        rowstride = width * channels;
+        if (rowstride / channels != width || rowstride + 3 < 0) /* overflow */
+                return NULL;
+        
+	/* Always align rows to 32-bit boundaries */
+	rowstride = (rowstride + 3) & ~3;
 
-	buf = g_try_malloc (height * rowstride);
+        bytes = height * rowstride;
+        if (bytes / rowstride !=  height) /* overflow */
+                return NULL;
+            
+	buf = g_try_malloc (bytes);
 	if (!buf)
 		return NULL;
 
@@ -170,7 +185,7 @@ gdk_pixbuf_new (GdkColorspace colorspace, gboolean has_alpha, int bits_per_sampl
  * Creates a new #GdkPixbuf with a copy of the information in the specified
  * @pixbuf.
  * 
- * Return value: A newly-created pixbuf with a reference count of 1, or NULL if
+ * Return value: A newly-created pixbuf with a reference count of 1, or %NULL if
  * not enough memory could be allocated.
  **/
 GdkPixbuf *
@@ -248,7 +263,7 @@ gdk_pixbuf_new_subpixbuf (GdkPixbuf *src_pixbuf,
                                         NULL, NULL);
 
         /* Keep a reference to src_pixbuf */
-        g_object_ref (G_OBJECT (src_pixbuf));
+        g_object_ref (src_pixbuf);
   
         g_object_set_qdata_full (G_OBJECT (sub),
                                  g_quark_from_static_string ("gdk-pixbuf-subpixbuf-src"),
@@ -300,7 +315,7 @@ gdk_pixbuf_get_n_channels (const GdkPixbuf *pixbuf)
  *
  * Queries whether a pixbuf has an alpha channel (opacity information).
  *
- * Return value: TRUE if it has an alpha channel, FALSE otherwise.
+ * Return value: %TRUE if it has an alpha channel, %FALSE otherwise.
  **/
 gboolean
 gdk_pixbuf_get_has_alpha (const GdkPixbuf *pixbuf)
@@ -413,7 +428,8 @@ gdk_pixbuf_error_quark (void)
 /**
  * gdk_pixbuf_fill:
  * @pixbuf: a #GdkPixbuf
- * @pixel: RGBA pixel to clear to (0xffffffff is opaque white, 0x00000000 transparent black)
+ * @pixel: RGBA pixel to clear to
+ *         (0xffffffff is opaque white, 0x00000000 transparent black)
  *
  * Clears a pixbuf to the given RGBA value, converting the RGBA value into
  * the pixbuf's pixel format. The alpha will be ignored if the pixbuf
@@ -425,13 +441,14 @@ gdk_pixbuf_fill (GdkPixbuf *pixbuf,
                  guint32    pixel)
 {
         guchar *pixels;
-        gboolean all_the_same = FALSE;
         guint r, g, b, a;
-        
+        guchar *p;
+        guint w, h;
+
         g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
 
         if (pixbuf->width == 0 || pixbuf->height == 0)
-          return;
+                return;
 
         pixels = pixbuf->pixels;
 
@@ -440,43 +457,35 @@ gdk_pixbuf_fill (GdkPixbuf *pixbuf,
         b = (pixel & 0x0000ff00) >> 8;
         a = (pixel & 0x000000ff);
 
-        if (r == g && g == b) {
-                if (!pixbuf->has_alpha)
-                        all_the_same = TRUE;
-                else
-                        all_the_same = (r == a);
-        }
+        h = pixbuf->height;
         
-        if (all_the_same) {
-                memset (pixels, r,
-                        pixbuf->rowstride * pixbuf->height);
-        } else {
-                guchar *p;
-                guchar  c[4];
-                gint    n;
-
-                c[0] = r; c[1] = g; c[2] = b; c[3] = a;
-                
+        while (h--) {
+                w = pixbuf->width;
                 p = pixels;
-                n = pixbuf->width;
-                if (pixbuf->has_alpha) {
-                        do {
-                                memcpy (p, c, 4);
-                                p += 4;
-                        } while (--n);
-                } else {
-                        do {
-                                memcpy (p, c, 3);
+
+                switch (pixbuf->n_channels) {
+                case 3:
+                        while (w--) {
+                                p[0] = r;
+                                p[1] = g;
+                                p[2] = b;
                                 p += 3;
-                        } while (--n);
-                }
-
-                p = pixels;
-                n = pixbuf->height - 1;
-                while (n--) {
-                        p += pixbuf->rowstride;
-                        memcpy (p, pixels, pixbuf->width * pixbuf->n_channels);
+                        }
+                        break;
+                case 4:
+                        while (w--) {
+                                p[0] = r;
+                                p[1] = g;
+                                p[2] = b;
+                                p[3] = a;
+                                p += 4;
+                        }
+                        break;
+                default:
+                        break;
                 }
+                
+                pixels += pixbuf->rowstride;
         }
 }
 
