@@ -1,39 +1,11 @@
-/* GDK - The GIMP Drawing Kit
- * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
- *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- */
-
-/*
- * Modified by the GTK+ Team and others 1997-2000.  See the AUTHORS
- * file for a list of people on the GTK+ Team.  See the ChangeLog
- * files for a list of changes.  These files are distributed with
- * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
- */
-
-#if defined(GTK_DISABLE_SINGLE_INCLUDES) && !defined (__GDK_H_INSIDE__) && !defined (GDK_COMPILATION)
-#error "Only <gdk/gdk.h> can be included directly."
-#endif
-
 #ifndef __GDK_DND_H__
 #define __GDK_DND_H__
 
 #include <gdk/gdktypes.h>
 
-G_BEGIN_DECLS
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
 
 typedef struct _GdkDragContext        GdkDragContext;
 
@@ -116,13 +88,13 @@ void             gdk_drag_context_unref      (GdkDragContext *context);
 
 void             gdk_drag_status        (GdkDragContext   *context,
 				         GdkDragAction     action,
-					 guint32           time_);
+					 guint32           time);
 void             gdk_drop_reply         (GdkDragContext   *context,
 					 gboolean          ok,
-					 guint32           time_);
+					 guint32           time);
 void             gdk_drop_finish        (GdkDragContext   *context,
 					 gboolean          success,
-					 guint32           time_);
+					 guint32           time);
 GdkAtom          gdk_drag_get_selection (GdkDragContext   *context);
 
 /* Source side */
@@ -130,10 +102,9 @@ GdkAtom          gdk_drag_get_selection (GdkDragContext   *context);
 GdkDragContext * gdk_drag_begin      (GdkWindow      *window,
 				      GList          *targets);
 
-GdkNativeWindow gdk_drag_get_protocol_for_display (GdkDisplay       *display,
-						   GdkNativeWindow   xid,
-						   GdkDragProtocol  *protocol);
-
+guint32 gdk_drag_get_protocol_for_display (GdkDisplay       *display,
+					   guint32           xid,
+					   GdkDragProtocol  *protocol);
 void    gdk_drag_find_window_for_screen   (GdkDragContext   *context,
 					   GdkWindow        *drag_window,
 					   GdkScreen        *screen,
@@ -143,9 +114,8 @@ void    gdk_drag_find_window_for_screen   (GdkDragContext   *context,
 					   GdkDragProtocol  *protocol);
 
 #ifndef GDK_MULTIHEAD_SAFE
-GdkNativeWindow gdk_drag_get_protocol (GdkNativeWindow   xid,
-				       GdkDragProtocol  *protocol);
-
+guint32 gdk_drag_get_protocol (guint32           xid,
+			       GdkDragProtocol  *protocol);
 void    gdk_drag_find_window  (GdkDragContext   *context,
 			       GdkWindow        *drag_window,
 			       gint              x_root,
@@ -161,13 +131,14 @@ gboolean        gdk_drag_motion      (GdkDragContext *context,
 				      gint            y_root,
 				      GdkDragAction   suggested_action,
 				      GdkDragAction   possible_actions,
-				      guint32         time_);
+				      guint32         time);
 void            gdk_drag_drop        (GdkDragContext *context,
-				      guint32         time_);
+				      guint32         time);
 void            gdk_drag_abort       (GdkDragContext *context,
-				      guint32         time_);
-gboolean        gdk_drag_drop_succeeded (GdkDragContext *context);
+				      guint32         time);
 
-G_END_DECLS
+#ifdef __cplusplus
+}
+#endif /* __cplusplus */
 
 #endif /* __GDK_DND_H__ */
