@@ -30,6 +30,7 @@
 #include "gdkpixmap.h"
 #include "gdkdrawable.h"
 #include "gdkpixmap.h"
+#include "gdkscreen.h"
 
 #define USE_BACKING_STORE	/* Appears to work on Win32, too, now. */
 
@@ -43,26 +44,19 @@ struct _GdkWindowPaint
   gint y_offset;
 };
 
-static const GdkPointerHooks default_pointer_hooks = {
-  _gdk_windowing_window_get_pointer,
-  _gdk_windowing_window_at_pointer
-};
-
-static const GdkPointerHooks *current_pointer_hooks = &default_pointer_hooks;
-
 static GdkGC *gdk_window_create_gc      (GdkDrawable     *drawable,
                                          GdkGCValues     *values,
                                          GdkGCValuesMask  mask);
 static void   gdk_window_draw_rectangle (GdkDrawable     *drawable,
                                          GdkGC           *gc,
-                                         gint             filled,
+                                         gboolean         filled,
                                          gint             x,
                                          gint             y,
                                          gint             width,
                                          gint             height);
 static void   gdk_window_draw_arc       (GdkDrawable     *drawable,
                                          GdkGC           *gc,
-                                         gint             filled,
+                                         gboolean         filled,
                                          gint             x,
                                          gint             y,
                                          gint             width,
@@ -71,7 +65,7 @@ static void   gdk_window_draw_arc       (GdkDrawable     *drawable,
                                          gint             angle2);
 static void   gdk_window_draw_polygon   (GdkDrawable     *drawable,
                                          GdkGC           *gc,
-                                         gint             filled,
+                                         gboolean         filled,
                                          GdkPoint        *points,
                                          gint             npoints);
 static void   gdk_window_draw_text      (GdkDrawable     *drawable,
@@ -154,8 +148,9 @@ static void   gdk_window_real_get_size  (GdkDrawable     *drawable,
 
 static GdkVisual*   gdk_window_real_get_visual   (GdkDrawable *drawable);
 static gint         gdk_window_real_get_depth    (GdkDrawable *drawable);
+static GdkScreen*   gdk_window_real_get_screen   (GdkDrawable *drawable);
 static void         gdk_window_real_set_colormap (GdkDrawable *drawable,
-                                             GdkColormap *cmap);
+						  GdkColormap *cmap);
 static GdkColormap* gdk_window_real_get_colormap (GdkDrawable *drawable);
 
 static GdkDrawable* gdk_window_get_composite_drawable (GdkDrawable *drawable,
@@ -238,8 +233,9 @@ gdk_window_class_init (GdkWindowObjectClass *klass)
   drawable_class->draw_lines = gdk_window_draw_lines;
   drawable_class->draw_glyphs = gdk_window_draw_glyphs;
   drawable_class->draw_image = gdk_window_draw_image;
-  drawable_class->_draw_pixbuf = gdk_window_draw_pixbuf;
+  drawable_class->draw_pixbuf = gdk_window_draw_pixbuf;
   drawable_class->get_depth = gdk_window_real_get_depth;
+  drawable_class->get_screen = gdk_window_real_get_screen;
   drawable_class->get_size = gdk_window_real_get_size;
   drawable_class->set_colormap = gdk_window_real_set_colormap;
   drawable_class->get_colormap = gdk_window_real_get_colormap;
@@ -270,7 +266,7 @@ gdk_window_finalize (GObject *object)
 	_gdk_window_destroy (window, TRUE);
     }
 
-  g_object_unref (G_OBJECT (obj->impl));
+  g_object_unref (obj->impl);
   obj->impl = NULL;
   
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -332,7 +328,7 @@ _gdk_window_destroy_hierarchy (GdkWindow *window,
               private->bg_pixmap != GDK_PARENT_RELATIVE_BG &&
               private->bg_pixmap != GDK_NO_BG)
 	    {
-	      gdk_pixmap_unref (private->bg_pixmap);
+	      g_object_unref (private->bg_pixmap);
 	      private->bg_pixmap = NULL;
 	    }
 	  
@@ -406,20 +402,19 @@ _gdk_window_destroy (GdkWindow *window,
  * gdk_window_destroy:
  * @window: a #GdkWindow
  *
- * Destroys @window (destroys the server-side resource associated with
- * @window).  Memory allocated for @window may not be freed until all
- * references to @window are dropped. All children of @window are also
- * destroyed.
+ * Destroys the window system resources associated with @window and decrements @window's
+ * reference count. The window system resources for all children of @window are also
+ * destroyed, but the children's reference counts are not decremented.
+ *
+ * Note that a window will not be destroyed automatically when its reference count
+ * reaches zero. You must call this function yourself before that happens.
  *
- * There's normally no need to use this function, window are automatically
- * destroyed when their reference count reaches 0.
- * 
  **/
 void
 gdk_window_destroy (GdkWindow *window)
 {
   _gdk_window_destroy_hierarchy (window, FALSE, FALSE);
-  gdk_drawable_unref (window);
+  g_object_unref (window);
 }
 
 /**
@@ -697,10 +692,46 @@ gdk_window_remove_filter (GdkWindow     *window,
     }
 }
 
+/**
+ * gdk_screen_get_toplevel_windows:
+ * @screen : The #GdkScreen where the toplevels are located.
+ * 
+ * Obtains a list of all toplevel windows known to GDK on the screen @screen.
+ * A toplevel window is a child of the root window (see
+ * gdk_get_default_root_window()).
+ *
+ * The returned list should be freed with g_list_free(), but
+ * its elements need not be freed.
+ * 
+ * Return value: list of toplevel windows, free with g_list_free()
+ **/
+GList *
+gdk_screen_get_toplevel_windows (GdkScreen *screen)
+{
+  GdkWindow * root_window;
+  GList *new_list = NULL;
+  GList *tmp_list;
+  
+  g_return_val_if_fail (GDK_IS_SCREEN (screen), NULL);
+  
+  root_window = gdk_screen_get_root_window (screen);
+
+  tmp_list = ((GdkWindowObject *)root_window)->children;
+  while (tmp_list)
+    {
+      if (GDK_WINDOW_TYPE (tmp_list->data) != GDK_WINDOW_FOREIGN)
+	new_list = g_list_prepend (new_list, tmp_list->data);
+      tmp_list = tmp_list->next;
+    }
+  
+  return new_list;
+}
+
 /**
  * gdk_window_get_toplevels:
  * 
- * Obtains a list of all toplevel windows known to GDK.
+ * Obtains a list of all toplevel windows known to GDK on the default
+ * screen (see gdk_window_get_toplevels_for_screen()).
  * A toplevel window is a child of the root window (see
  * gdk_get_default_root_window()).
  *
@@ -712,18 +743,7 @@ gdk_window_remove_filter (GdkWindow     *window,
 GList *
 gdk_window_get_toplevels (void)
 {
-  GList *new_list = NULL;
-  GList *tmp_list;
-  
-  tmp_list = ((GdkWindowObject *)_gdk_parent_root)->children;
-  while (tmp_list)
-    {
-      if (GDK_WINDOW_TYPE (tmp_list->data) != GDK_WINDOW_FOREIGN)
-	new_list = g_list_prepend (new_list, tmp_list->data);
-      tmp_list = tmp_list->next;
-    }
-  
-  return new_list;
+  return gdk_screen_get_toplevel_windows (gdk_screen_get_default ());
 }
 
 /**
@@ -758,12 +778,14 @@ gboolean
 gdk_window_is_viewable (GdkWindow *window)
 {
   GdkWindowObject *private = (GdkWindowObject *)window;
+  GdkScreen *screen = gdk_drawable_get_screen (window);
+  GdkWindow *root_window = gdk_screen_get_root_window (screen);
   
   g_return_val_if_fail (window != NULL, FALSE);
   g_return_val_if_fail (GDK_IS_WINDOW (window), FALSE);
   
   while (private && 
-	 (private != (GdkWindowObject *)_gdk_parent_root) &&
+	 (private != (GdkWindowObject *)root_window) &&
 	 (GDK_WINDOW_TYPE (private) != GDK_WINDOW_FOREIGN))
     {
       if (!GDK_WINDOW_IS_MAPPED (window))
@@ -870,7 +892,7 @@ gdk_window_paint_init_bg (GdkWindow      *window,
   gdk_gc_set_clip_region (tmp_gc, init_region);
 
   gdk_draw_rectangle (paint->pixmap, tmp_gc, TRUE, 0, 0, -1, -1);
-  gdk_gc_unref (tmp_gc);
+  g_object_unref (tmp_gc);
 }
 
 #ifdef GDK_WINDOWING_X11
@@ -970,8 +992,8 @@ gdk_window_begin_paint_region (GdkWindow *window,
 			     0, 0,
 			     old_rect.x - new_rect.x, old_rect.y - new_rect.y,
                              old_rect.width, old_rect.height);
-          gdk_gc_unref (tmp_gc);
-	  gdk_drawable_unref (tmp_paint->pixmap);
+          g_object_unref (tmp_gc);
+	  g_object_unref (tmp_paint->pixmap);
 
 	  paint->x_offset = new_rect.x;
 	  paint->y_offset = new_rect.y;
@@ -1075,7 +1097,7 @@ gdk_window_end_paint (GdkWindow *window)
                      clip_box.y - paint->y_offset,
                      clip_box.x - x_offset, clip_box.y - y_offset,
                      clip_box.width, clip_box.height);
-  gdk_gc_unref (tmp_gc);
+  g_object_unref (tmp_gc);
 
   if (private->paint_stack)
     {
@@ -1089,7 +1111,7 @@ gdk_window_end_paint (GdkWindow *window)
 	}
     }
   else
-    gdk_drawable_unref (paint->pixmap);
+    g_object_unref (paint->pixmap);
 
   gdk_region_destroy (paint->region);
   g_free (paint);
@@ -1110,7 +1132,7 @@ gdk_window_free_paint_stack (GdkWindow *window)
 	  GdkWindowPaint *paint = tmp_list->data;
 
 	  if (tmp_list == private->paint_stack)
-	    gdk_drawable_unref (paint->pixmap);
+	    g_object_unref (paint->pixmap);
 		  
 	  gdk_region_destroy (paint->region);
 	  g_free (paint);
@@ -1235,7 +1257,7 @@ gdk_window_create_gc (GdkDrawable     *drawable,
 static void
 gdk_window_draw_rectangle (GdkDrawable *drawable,
 			   GdkGC       *gc,
-			   gint         filled,
+			   gboolean     filled,
 			   gint         x,
 			   gint         y,
 			   gint         width,
@@ -1263,7 +1285,7 @@ gdk_window_draw_rectangle (GdkDrawable *drawable,
 static void
 gdk_window_draw_arc (GdkDrawable *drawable,
 		     GdkGC       *gc,
-		     gint         filled,
+		     gboolean     filled,
 		     gint         x,
 		     gint         y,
 		     gint         width,
@@ -1294,7 +1316,7 @@ gdk_window_draw_arc (GdkDrawable *drawable,
 static void
 gdk_window_draw_polygon (GdkDrawable *drawable,
 			 GdkGC       *gc,
-			 gint         filled,
+			 gboolean     filled,
 			 GdkPoint    *points,
 			 gint         npoints)
 {
@@ -1420,7 +1442,7 @@ gdk_window_get_composite_drawable (GdkDrawable *window,
                                          composite_x_offset,
                                          composite_y_offset);
       
-      return GDK_DRAWABLE (g_object_ref (G_OBJECT (window)));
+      return g_object_ref (window);
     }
   
   buffered_region = NULL;
@@ -1470,7 +1492,7 @@ gdk_window_get_composite_drawable (GdkDrawable *window,
                                          composite_x_offset,
                                          composite_y_offset);
 
-      return GDK_DRAWABLE (g_object_ref (G_OBJECT (window)));
+      return g_object_ref (window);
     }
   
   tmp_pixmap = gdk_pixmap_new (window,
@@ -1519,7 +1541,7 @@ gdk_window_get_composite_drawable (GdkDrawable *window,
   *composite_x_offset = x;
   *composite_y_offset = y;
 
-  g_object_unref (G_OBJECT (tmp_gc));
+  g_object_unref (tmp_gc);
   
   return tmp_pixmap;
 }
@@ -1770,7 +1792,7 @@ gdk_window_clear_backing_rect (GdkWindow *window,
   tmp_gc = gdk_window_get_bg_gc (window, paint);
   gdk_draw_rectangle (paint->pixmap, tmp_gc, TRUE,
 		      x - paint->x_offset, y - paint->y_offset, width, height);
-  gdk_gc_unref (tmp_gc);
+  g_object_unref (tmp_gc);
 }
 
 /**
@@ -1915,16 +1937,16 @@ gdk_window_draw_pixbuf (GdkDrawable     *drawable,
       if (private->paint_stack)
 	{
 	  GdkWindowPaint *paint = private->paint_stack->data;
-	  _gdk_draw_pixbuf (paint->pixmap, gc, pixbuf, src_x, src_y,
-			    dest_x - x_offset, dest_y - y_offset,
-			    width, height,
-			    dither, x_dither - x_offset, y_dither - y_offset);
+	  gdk_draw_pixbuf (paint->pixmap, gc, pixbuf, src_x, src_y,
+			   dest_x - x_offset, dest_y - y_offset,
+			   width, height,
+			   dither, x_dither - x_offset, y_dither - y_offset);
 	}
       else
-	_gdk_draw_pixbuf (private->impl, gc, pixbuf, src_x, src_y,
-			  dest_x - x_offset, dest_y - y_offset,
-			  width, height,
-			  dither, x_dither, y_dither);
+	gdk_draw_pixbuf (private->impl, gc, pixbuf, src_x, src_y,
+			 dest_x - x_offset, dest_y - y_offset,
+			 width, height,
+			 dither, x_dither, y_dither);
       
       RESTORE_GC (gc);
     }
@@ -1936,16 +1958,16 @@ gdk_window_draw_pixbuf (GdkDrawable     *drawable,
       if (private->paint_stack)
 	{
 	  GdkWindowPaint *paint = private->paint_stack->data;
-	  _gdk_draw_pixbuf (paint->pixmap, gc, pixbuf, src_x, src_y,
-			    dest_x - x_offset, dest_y - y_offset,
-			    width, height,
+	  gdk_draw_pixbuf (paint->pixmap, gc, pixbuf, src_x, src_y,
+			   dest_x - x_offset, dest_y - y_offset,
+			   width, height,
 			    dither, x_dither - x_offset, y_dither - y_offset);
 	}
       else
-	_gdk_draw_pixbuf (private->impl, gc, pixbuf, src_x, src_y,
-			  dest_x - x_offset, dest_y - y_offset,
-			  width, height,
-			  dither, x_dither, y_dither);
+	gdk_draw_pixbuf (private->impl, gc, pixbuf, src_x, src_y,
+			 dest_x - x_offset, dest_y - y_offset,
+			 width, height,
+			 dither, x_dither, y_dither);
     }
 }
 
@@ -1974,19 +1996,15 @@ gdk_window_real_get_visual (GdkDrawable *drawable)
 static gint
 gdk_window_real_get_depth (GdkDrawable *drawable)
 {
-  gint depth;
-  
   g_return_val_if_fail (GDK_IS_WINDOW (drawable), 0);
 
-  depth = ((GdkWindowObject *)GDK_WINDOW (drawable))->depth;
+  return ((GdkWindowObject *)GDK_WINDOW (drawable))->depth;
+}
 
-  if (depth == 0)
-    {
-      g_print ("0 depth for type %s\n", g_type_name (G_OBJECT_TYPE (drawable)));
-      G_BREAKPOINT ();
-    }
-
-  return depth;
+static GdkScreen*
+gdk_window_real_get_screen (GdkDrawable *drawable)
+{
+  return gdk_drawable_get_screen (GDK_WINDOW_OBJECT (drawable)->impl);
 }
 
 static void
@@ -2065,7 +2083,8 @@ gdk_window_process_updates_internal (GdkWindow *window)
       GdkRegion *update_area = private->update_area;
       private->update_area = NULL;
       
-      if (_gdk_event_func && gdk_window_is_viewable (window))
+      if (_gdk_event_func && gdk_window_is_viewable (window) &&
+	  (private->event_mask & GDK_EXPOSURE_MASK))
 	{
 	  GdkEvent event;
 	  GdkRectangle window_rect;
@@ -2076,7 +2095,7 @@ gdk_window_process_updates_internal (GdkWindow *window)
           if (debug_updates)
             {
               /* Make sure we see the red invalid area before redrawing. */
-              gdk_flush ();
+              gdk_display_sync (gdk_drawable_get_display (window));
               g_usleep (70000);
             }
           
@@ -2090,7 +2109,7 @@ gdk_window_process_updates_internal (GdkWindow *window)
 	  save_region = _gdk_windowing_window_queue_antiexpose (window, update_area);
       
 	  event.expose.type = GDK_EXPOSE;
-	  event.expose.window = gdk_window_ref (window);
+	  event.expose.window = g_object_ref (window);
 	  event.expose.count = 0;
 
 	  if (save_region)
@@ -2112,7 +2131,7 @@ gdk_window_process_updates_internal (GdkWindow *window)
 
 	  if (expose_region != update_area)
 	    gdk_region_destroy (expose_region);
-	  gdk_window_unref (window);
+	  g_object_unref (window);
 	}
       if (!save_region)
 	gdk_region_destroy (update_area);
@@ -2138,9 +2157,12 @@ gdk_window_process_all_updates (void)
   update_windows = NULL;
   update_idle = 0;
 
+  g_slist_foreach (old_update_windows, (GFunc)g_object_ref, NULL);
+  
   while (tmp_list)
     {
       gdk_window_process_updates_internal (tmp_list->data);
+      g_object_unref (tmp_list->data);
       tmp_list = tmp_list->next;
     }
 
@@ -2152,7 +2174,9 @@ gdk_window_process_all_updates (void)
 static gboolean
 gdk_window_update_idle (gpointer data)
 {
+  GDK_THREADS_ENTER ();
   gdk_window_process_all_updates ();
+  GDK_THREADS_LEAVE ();
   
   return FALSE;
 }
@@ -2242,6 +2266,30 @@ gdk_window_invalidate_rect   (GdkWindow    *window,
   gdk_region_destroy (region);
 }
 
+static void
+draw_ugly_color (GdkWindow *window,
+                GdkRegion *region)
+{
+  /* Draw ugly color all over the newly-invalid region */
+  GdkColor ugly_color = { 0, 50000, 10000, 10000 };
+  GdkGC *ugly_gc;
+  GdkRectangle clipbox;
+    
+  ugly_gc = gdk_gc_new (window);
+  gdk_gc_set_rgb_fg_color (ugly_gc, &ugly_color);
+  gdk_gc_set_clip_region (ugly_gc, region);
+
+  gdk_region_get_clipbox (region, &clipbox);
+  
+  gdk_draw_rectangle (window,
+		      ugly_gc,
+		      TRUE,
+		      clipbox.x, clipbox.y,
+		      clipbox.width, clipbox.height);
+  
+  g_object_unref (ugly_gc);
+}
+
 /**
  * gdk_window_invalidate_maybe_recurse:
  * @window: a #GdkWindow
@@ -2291,26 +2339,7 @@ gdk_window_invalidate_maybe_recurse (GdkWindow *window,
   if (!gdk_region_empty (visible_region))
     {
       if (debug_updates)
-        {
-          /* Draw ugly color all over the newly-invalid region */
-          GdkRectangle ugly_rect;
-          GdkGC *ugly_gc;
-          GdkColor ugly_color = { 0, 60000, 10000, 10000 };
-          
-          ugly_gc = gdk_gc_new (window);
-
-          gdk_gc_set_rgb_fg_color (ugly_gc, &ugly_color);
-          
-	  gdk_region_get_clipbox (visible_region, &ugly_rect);
-
-          gdk_draw_rectangle (window,
-                              ugly_gc,
-                              TRUE,
-                              ugly_rect.x, ugly_rect.y,
-                              ugly_rect.width, ugly_rect.height);
-          
-          g_object_unref (G_OBJECT (ugly_gc));
-        }
+        draw_ugly_color (window, region);
       
       if (private->update_area)
 	{
@@ -2661,33 +2690,6 @@ gdk_window_constrain_size (GdkGeometry *geometry,
   *new_height = height;
 }
 
-/**
- * gdk_set_pointer_hooks:
- * @new_hooks: a table of pointers to functions for getting
- *   quantities related to the current pointer position,
- *   or %NULL to restore the default table.
- * 
- * This function allows for hooking into the operation
- * of getting the current location of the pointer. This
- * is only useful for such low-level tools as an
- * event recorder. Applications should never have any
- * reason to use this facility
- * 
- * Return value: the previous pointer hook table
- **/
-GdkPointerHooks *
-gdk_set_pointer_hooks (const GdkPointerHooks *new_hooks)
-{
-  const GdkPointerHooks *result = current_pointer_hooks;
-
-  if (new_hooks)
-    current_pointer_hooks = new_hooks;
-  else
-    current_pointer_hooks = &default_pointer_hooks;
-
-  return (GdkPointerHooks *)result;
-}
-
 /**
  * gdk_window_get_pointer:
  * @window: a #GdkWindow
@@ -2706,11 +2708,41 @@ GdkWindow*
 gdk_window_get_pointer (GdkWindow	  *window,
 			gint		  *x,
 			gint		  *y,
-			GdkModifierType *mask)
+			GdkModifierType   *mask)
 {
-  g_return_val_if_fail (window == NULL || GDK_IS_WINDOW (window), NULL);
+  GdkDisplay *display;
+  gint tmp_x, tmp_y;
+  GdkModifierType tmp_mask;
+  GdkWindow *child;
   
-  return current_pointer_hooks->get_pointer (window, x, y, mask); 
+  g_return_val_if_fail (window == NULL || GDK_IS_WINDOW (window), NULL);
+
+  if (window)
+    {
+      display = gdk_drawable_get_display (window);
+    }
+  else
+    {
+      GdkScreen *screen = gdk_screen_get_default ();
+
+      display = gdk_screen_get_display (screen);
+      window = gdk_screen_get_root_window (screen);
+      
+      GDK_NOTE (MULTIHEAD,
+		g_message ("Passing NULL for window to gdk_window_get_pointer()\n"
+			   "is not multihead safe"));
+    }
+
+  child = display->pointer_hooks->window_get_pointer (display, window, &tmp_x, &tmp_y, &tmp_mask);
+
+  if (x)
+    *x = tmp_x;
+  if (y)
+    *y = tmp_y;
+  if (mask)
+    *mask = tmp_mask;
+
+  return child;
 }
 
 /**
@@ -2720,8 +2752,12 @@ gdk_window_get_pointer (GdkWindow	  *window,
  * 
  * Obtains the window underneath the mouse pointer, returning the
  * location of that window in @win_x, @win_y. Returns %NULL if the
- * window under the mouse pointer is not known to GDK (for example,
- * belongs to another application).
+ * window under the mouse pointer is not known to GDK (if the window
+ * belongs to another application and a #GdkWindow hasn't been created
+ * for it with gdk_window_foreign_new())
+ *
+ * NOTE: For multihead-aware widgets or applications use
+ * gdk_display_get_window_at_pointer() instead.
  * 
  * Return value: window under the mouse pointer
  **/
@@ -2729,7 +2765,7 @@ GdkWindow*
 gdk_window_at_pointer (gint *win_x,
 		       gint *win_y)
 {
-  return current_pointer_hooks->window_at_pointer (NULL, win_x, win_y);
+  return gdk_display_get_window_at_pointer (gdk_display_get_default (), win_x, win_y);
 }
 
 /**
@@ -2743,6 +2779,25 @@ gdk_window_at_pointer (gint *win_x,
 GdkWindow *
 gdk_get_default_root_window (void)
 {
-  return _gdk_parent_root;
+  return gdk_screen_get_root_window (gdk_screen_get_default ());
+}
+
+/**
+ * gdk_window_foreign_new:
+ * @anid: a native window handle.
+ * 
+ * Wraps a native window for the default display in a #GdkWindow.
+ * This may fail if the window has been destroyed.
+ *
+ * For example in the X backend, a native window handle is an Xlib
+ * <type>XID</type>.
+ * 
+ * Return value: the newly-created #GdkWindow wrapper for the 
+ *    native window or %NULL if the window has been destroyed.
+ **/
+GdkWindow *
+gdk_window_foreign_new (GdkNativeWindow anid)
+{
+  return gdk_window_foreign_new_for_display (gdk_display_get_default (), anid);
 }
 
