@@ -147,7 +147,7 @@ static GdkFBKeyboardDevice keyb_devs[] =
 };
 
 GdkKeymap*
-gdk_keymap_get_default (void)
+gdk_keymap_get_default_for_display (GdkDisplay *display)
 {
   if (default_keymap == NULL)
     default_keymap = g_object_new (gdk_keymap_get_type (), NULL);
@@ -155,6 +155,12 @@ gdk_keymap_get_default (void)
   return default_keymap;
 }
 
+GdkKeymap*
+gdk_keymap_get_for_display (GdkDisplay *display)
+{
+  return gdk_keymap_get_default_for_display (display);
+}
+
 PangoDirection
 gdk_keymap_get_direction (GdkKeymap *keymap)
 {
@@ -186,7 +192,7 @@ gdk_fb_keyboard_init (gboolean open_dev)
 
   for (i = 0; i < G_N_ELEMENTS(keyb_devs); i++)
     {
-      if (g_strcasecmp(keyb_type, keyb_devs[i].name)==0)
+      if (g_ascii_strcasecmp(keyb_type, keyb_devs[i].name)==0)
 	break;
     }
   
@@ -229,27 +235,6 @@ gdk_fb_keyboard_close (void)
 }
 
 
-/**
- * gdk_keymap_get_entries_for_keyval:
- * @keymap: a #GdkKeymap, or %NULL to use the default keymap
- * @keyval: a keyval, such as %GDK_a, %GDK_Up, %GDK_Return, etc.
- * @keys: return location for an array of #GdkKeymapKey
- * @n_keys: return location for number of elements in returned array
- * 
- * Obtains a list of keycode/group/level combinations that will
- * generate @keyval. Groups and levels are two kinds of keyboard mode;
- * in general, the level determines whether the top or bottom symbol
- * on a key is used, and the group determines whether the left or
- * right symbol is used. On US keyboards, the shift key changes the
- * keyboard level, and there are no groups. A group switch key might
- * convert a keyboard between Hebrew to English modes, for example.
- * #GdkEventKey contains a %group field that indicates the active
- * keyboard group. The level is computed from the modifier mask.
- * The returned array should be freed
- * with g_free().
- *
- * Return value: %TRUE if keys were found and returned
- **/
 gboolean
 gdk_keymap_get_entries_for_keyval (GdkKeymap     *keymap,
                                    guint          keyval,
@@ -267,23 +252,6 @@ gdk_keymap_get_entries_for_keyval (GdkKeymap     *keymap,
 						       n_keys);
 }
 
-/**
- * gdk_keymap_get_entries_for_keycode:
- * @keymap: a #GdkKeymap or %NULL to use the default keymap
- * @hardware_keycode: a keycode
- * @keys: return location for array of #GdkKeymapKey, or NULL
- * @keyvals: return location for array of keyvals, or NULL
- * @n_entries: length of @keys and @keyvals
- *
- * Returns the keyvals bound to @hardware_keycode.
- * The Nth #GdkKeymapKey in @keys is bound to the Nth
- * keyval in @keyvals. Free the returned arrays with g_free().
- * When a keycode is pressed by the user, the keyval from
- * this list of entries is selected by considering the effective
- * keyboard group and level. See gdk_keymap_translate_keyboard_state().
- *
- * Returns: %TRUE if there were any entries
- **/
 gboolean
 gdk_keymap_get_entries_for_keycode (GdkKeymap     *keymap,
                                     guint          hardware_keycode,
@@ -302,19 +270,6 @@ gdk_keymap_get_entries_for_keycode (GdkKeymap     *keymap,
 }
 
 
-/**
- * gdk_keymap_lookup_key:
- * @keymap: a #GdkKeymap or %NULL to use the default keymap
- * @key: a #GdkKeymapKey with keycode, group, and level initialized
- * 
- * Looks up the keyval mapped to a keycode/group/level triplet.
- * If no keyval is bound to @key, returns 0. For normal user input,
- * you want to use gdk_keymap_translate_keyboard_state() instead of
- * this function, since the effective group/level may not be
- * the same as the current keyboard state.
- * 
- * Return value: a keyval, or 0 if none was mapped to the given @key
- **/
 guint
 gdk_keymap_lookup_key (GdkKeymap          *keymap,
                        const GdkKeymapKey *key)
@@ -327,30 +282,6 @@ gdk_keymap_lookup_key (GdkKeymap          *keymap,
 }
 
 
-/**
- * gdk_keymap_translate_keyboard_state:
- * @keymap: a #GdkKeymap, or %NULL to use the default
- * @hardware_keycode: a keycode
- * @state: a modifier state 
- * @group: active keyboard group
- * @keyval: return location for keyval
- * @effective_group: return location for effective group
- * @level: return location for level
- * @consumed_modifiers: return location for modifiers that were used to determine the group or level
- * 
- *
- * Translates the contents of a #GdkEventKey into a keyval, effective
- * group, and level. Modifiers that affected the translation and
- * are thus unavailable for application use are returned in
- * @consumed_modifiers.  See gdk_keyval_get_keys() for an explanation of
- * groups and levels.  The @effective_group is the group that was
- * actually used for the translation; some keys such as Enter are not
- * affected by the active keyboard group. The @level is derived from
- * @state. For convenience, #GdkEventKey already contains the translated
- * keyval, so this function isn't as useful as you might think.
- * 
- * Return value: %TRUE if there was a keyval bound to the keycode/state/group
- **/
 gboolean
 gdk_keymap_translate_keyboard_state (GdkKeymap       *keymap,
                                      guint            hardware_keycode,
@@ -388,7 +319,8 @@ gdk_fb_handle_key (guint hw_keycode,
 
   /* handle some magic keys */
   if (key_up &&
-      (modifier_state & (GDK_CONTROL_MASK|GDK_MOD1_MASK)))
+      (modifier_state & GDK_CONTROL_MASK) &&
+      (modifier_state & GDK_MOD1_MASK))
     {
       if (keyval == GDK_BackSpace)
 	{
@@ -777,7 +709,7 @@ xlate_io (GIOChannel *gioc,
   if (n <= 0)
     g_error ("Nothing from keyboard!");
 
-  for (i = 0; i < n; i++)
+  for (i = 0; i < n; )
     {
       handled = FALSE;
       modifier = 0;
@@ -810,6 +742,10 @@ xlate_io (GIOChannel *gioc,
 			  gdk_fb_set_rotation (deg);
 			}
 
+		      if ((xlate_codes[j].code == GDK_F8) &&
+			  (xlate_codes[j].modifier & GDK_SHIFT_MASK))
+			exit (1);
+
 		      
 		      gdk_fb_handle_key (xlate_codes[j].code,
 					 xlate_codes[j].code,
@@ -873,10 +809,10 @@ xlate_io (GIOChannel *gioc,
 }
 
 static gboolean
-write_string (gint   fd,
-	      gchar *str)
+write_string (gint         fd,
+	      const gchar *str)
 {
-  gsize to_write = strlen (string);
+  gsize to_write = strlen (str);
 
   while (to_write > 0)
     {
@@ -929,7 +865,7 @@ static void
 xlate_close (GdkFBKeyboard *kb)
 {
   struct termios ts;
-  const char cursoron_str[] = "\033c";
+  const char cursoron_str[] = "\033c\033[3g\033]R";
 
   write_string (gdk_display->tty_fd, cursoron_str);
 
@@ -947,8 +883,11 @@ static guint
 xlate_lookup (GdkFBKeyboard       *kb,
 	      const GdkKeymapKey  *key)
 {
-  g_warning ("xlate_lookup() NIY");
-  return FALSE;
+  if (key->group != 0)
+    return 0;
+  if (key->level != 0)
+    return 0;
+  return key->keycode;
 }
 
 static gboolean
@@ -961,8 +900,15 @@ xlate_translate (GdkFBKeyboard       *kb,
 		 gint                *level,
 		 GdkModifierType     *consumed_modifiers)
 {
-  g_warning ("xlate_translate() NIY");
-  return FALSE;
+  if (keyval)
+    *keyval = hardware_keycode;
+  if (effective_group)
+    *effective_group = 0;
+  if (level)
+    *level = 0;
+  if (consumed_modifiers)
+    *consumed_modifiers = 0;
+  return TRUE;
 }
 
 static gboolean
@@ -971,10 +917,12 @@ xlate_get_for_keyval (GdkFBKeyboard       *kb,
 		      GdkKeymapKey       **keys,
 		      gint                *n_keys)
 {
-  g_warning ("xlate_get_for_keyval() NIY");
-  *keys=NULL;
-  *n_keys=0;
-  return FALSE;
+  *n_keys = 1;
+  *keys = g_new (GdkKeymapKey, 1);
+  (*keys)->keycode = keyval;
+  (*keys)->group = 0;
+  (*keys)->level = 0;
+  return TRUE;
 }
 
 static gboolean
@@ -984,10 +932,22 @@ xlate_get_for_keycode (GdkFBKeyboard       *kb,
 		       guint              **keyvals,
 		       gint                *n_entries)
 {
-  g_warning ("xlate_get_for_keycode() NIY");
-  *keys=NULL;
-  *n_entries=0;
-  return FALSE;
+  if (keys)
+    {
+      *keys = g_new (GdkKeymapKey, 1);
+      (*keys)->keycode = hardware_keycode;
+      (*keys)->level = 0;
+      (*keys)->group = 0;
+    }
+  if (keyvals)
+    {
+      *keyvals = g_new (guint, 1);
+      **keyvals = hardware_keycode;
+    }
+  if (n_entries)
+    *n_entries = 1;
+
+  return TRUE;
 }
 
 /* Raw keyboard support */
@@ -1109,16 +1069,16 @@ static const guint trans_table[256][3] = {
   {0, 0, 0},
   {0, 0, 0},
   {0, 0, 0},
-  {0, 0, 0},
-  {0, 0, 0},
-  {0, 0, 0},
-  {0, 0, 0},
-  {0, 0, 0},
-  {0, 0, 0},
-  {0, 0, 0},
-  {0, 0, 0},
-  {0, 0, 0},
-  {0, 0, 0},
+  {GDK_Home, 0, 0},
+  {GDK_Up, 0, 0},
+  {GDK_Page_Up, 0, 0},
+  {GDK_Left, 0, 0},
+  {GDK_Right, 0, 0},
+  {GDK_End, 0, 0},
+  {GDK_Down, 0, 0},
+  {GDK_Page_Down, 0, 0},
+  {GDK_Insert, 0, 0},
+  {GDK_Delete, 0, 0},
 
 	/* 0x70 */
   {0, 0, 0},
@@ -1425,7 +1385,7 @@ raw_open (GdkFBKeyboard *kb)
   
   if (ioctl (gdk_display->tty_fd, KDSKBMODE, K_MEDIUMRAW) < 0)
     {
-      g_warning ("setting tty to K_MEDIUMRAW failed");
+      g_warning ("setting tty to K_MEDIUMRAW failed (are you root?)");
       return FALSE;
     }
 
@@ -1460,47 +1420,185 @@ raw_close (GdkFBKeyboard *kb)
 
 static guint
 raw_lookup (GdkFBKeyboard       *kb,
-	      const GdkKeymapKey  *key)
+	    const GdkKeymapKey  *key)
 {
-  g_warning ("raw_lookup() NIY");
-  return FALSE;
+  if (key->group != 0)
+    return 0;
+  if ((key->keycode < 0) || (key->keycode >= 256))
+    return 0;
+  if ((key->level < 0) || (key->level >= 3))
+    return 0;
+  return trans_table[key->keycode][key->level];
 }
 
 static gboolean
 raw_translate (GdkFBKeyboard       *kb,
-		 guint                hardware_keycode,
-		 GdkModifierType      state,
-		 gint                 group,
-		 guint               *keyval,
-		 gint                *effective_group,
-		 gint                *level,
-		 GdkModifierType     *consumed_modifiers)
+	       guint                hardware_keycode,
+	       GdkModifierType      state,
+	       gint                 group,
+	       guint               *keyval,
+	       gint                *effective_group,
+	       gint                *level,
+	       GdkModifierType     *consumed_modifiers)
 {
-  g_warning ("raw_translate() NIY");
-  return FALSE;
+  guint tmp_keyval;
+  gint tmp_level;
+
+  if (keyval)
+    *keyval = 0;
+  if (effective_group)
+    *effective_group = 0;
+  if (level)
+    *level = 0;
+  if (consumed_modifiers)
+    *consumed_modifiers = 0;
+
+  if ((hardware_keycode < 0) || (hardware_keycode >= 256))
+    return FALSE;
+
+  if (group != 0)
+    return FALSE;
+
+  if (state & GDK_CONTROL_MASK)
+    tmp_level = 2;
+  else if (state & GDK_SHIFT_MASK)
+    tmp_level = 1;
+  else
+    tmp_level = 0;
+
+  do {
+    tmp_keyval = trans_table[hardware_keycode][tmp_level --];
+  } while (!tmp_keyval && (tmp_level >= 0));
+
+  if (keyval)
+    *keyval = tmp_keyval;
+  if (level)
+    *level = tmp_level;
+
+  return TRUE;
 }
 
 static gboolean
 raw_get_for_keyval (GdkFBKeyboard       *kb,
-		      guint                keyval,
-		      GdkKeymapKey       **keys,
-		      gint                *n_keys)
+		    guint                keyval,
+		    GdkKeymapKey       **keys,
+		    gint                *n_keys)
 {
-  g_warning ("raw_get_for_keyval() NIY");
-  *keys=NULL;
-  *n_keys=0;
-  return FALSE;
+  GArray *retval;
+  int i, j;
+
+  retval = g_array_new (FALSE, FALSE, sizeof (GdkKeymapKey));
+
+  for (i = 0; i < 256; i++)
+    for (j = 0; j < 3; j++)
+      if (trans_table[i][j] == keyval)
+	{
+	  GdkKeymapKey key;
+
+	  key.keycode = i;
+	  key.group = 0;
+	  key.level = j;
+
+	  g_array_append_val (retval, key);
+	}
+
+  if (retval->len > 0)
+    {
+      *keys = (GdkKeymapKey*) retval->data;
+      *n_keys = retval->len;
+    }
+  else
+    {
+      *keys = NULL;
+      *n_keys = 0;
+    }
+
+  g_array_free (retval, retval->len > 0 ? FALSE : TRUE);
+
+  return *n_keys > 0;
 }
 
 static gboolean
 raw_get_for_keycode (GdkFBKeyboard       *kb,
-		       guint                hardware_keycode,
-		       GdkKeymapKey       **keys,
-		       guint              **keyvals,
-		       gint                *n_entries)
+		     guint                hardware_keycode,
+		     GdkKeymapKey       **keys,
+		     guint              **keyvals,
+		     gint                *n_entries)
 {
-  g_warning ("raw_get_for_keycode() NIY");
-  *keys=NULL;
-  *n_entries=0;
-  return FALSE;
+  GArray *key_array;
+  GArray *keyval_array;
+  int i;
+
+  if (hardware_keycode <= 0 ||
+      hardware_keycode >= 256)
+    { 
+      if (keys)
+        *keys = NULL;
+      if (keyvals)
+        *keyvals = NULL;
+
+      *n_entries = 0;
+      return FALSE;
+    }
+
+  if (keys)
+    key_array = g_array_new (FALSE, FALSE, sizeof (GdkKeymapKey));
+  else
+    key_array = NULL;
+
+  if (keyvals)
+    keyval_array = g_array_new (FALSE, FALSE, sizeof (guint));
+  else
+    keyval_array = NULL;
+
+  for (i = 0; i < 3; i++)
+    {
+      if (key_array)
+	{
+	  GdkKeymapKey key;
+
+	  key.keycode = hardware_keycode;
+	  key.group = 0;
+	  key.level = i;
+
+	  g_array_append_val (key_array, key);
+	}
+
+      if (keyval_array)
+	{
+	  g_array_append_val (keyval_array, trans_table[hardware_keycode][i]);
+	}
+    }
+
+  if ((key_array && key_array->len > 0) ||
+      (keyval_array && keyval_array->len > 0))
+    { 
+      if (keys)
+        *keys = (GdkKeymapKey*) key_array->data;
+
+      if (keyvals)
+        *keyvals = (guint*) keyval_array->data;
+
+      if (key_array)
+        *n_entries = key_array->len;
+      else
+        *n_entries = keyval_array->len;
+    }
+  else
+    { 
+      if (keys)
+        *keys = NULL;
+
+      if (keyvals)
+        *keyvals = NULL;
+
+      *n_entries = 0;
+    }
+
+  if (key_array)
+    g_array_free (key_array, key_array->len > 0 ? FALSE : TRUE);
+  if (keyval_array)
+    g_array_free (keyval_array, keyval_array->len > 0 ? FALSE : TRUE);
+
+  return *n_entries > 0;
 }
