@@ -118,7 +118,8 @@ enum {
   FB_VSYNC,
   FB_CSYNC,
   FB_EXTSYNC,
-  FB_DOUBLE
+  FB_DOUBLE,
+  FB_ACCEL
 };
 
 char *fb_modes_keywords[] =
@@ -132,7 +133,8 @@ char *fb_modes_keywords[] =
   "vsync",
   "csync",
   "extsync",
-  "double"
+  "double",
+  "accel"
 };
 
 static int
@@ -146,7 +148,7 @@ fb_modes_parse_mode (GScanner *scanner,
   char *modename;
   int geometry[5];
   int timings[7];
-  int vsync=0, hsync=0, csync=0, extsync=0, doublescan=0, laced=0;
+  int vsync=0, hsync=0, csync=0, extsync=0, doublescan=0, laced=0, accel=1;
   int found_geometry = 0;
   int found_timings = 0;
     
@@ -207,9 +209,9 @@ fb_modes_parse_mode (GScanner *scanner,
 		g_free (modename);
 		return -1;
 	      }
-	  if (g_strcasecmp (scanner->value.v_identifier, "true")==0)
+	  if (g_ascii_strcasecmp (scanner->value.v_identifier, "true")==0)
 	    laced = 1;
-	  else if (g_strcasecmp (scanner->value.v_identifier, "false")==0)
+	  else if (g_ascii_strcasecmp (scanner->value.v_identifier, "false")==0)
 	    laced = 0;
 	  else
 	    {
@@ -224,9 +226,9 @@ fb_modes_parse_mode (GScanner *scanner,
 		g_free (modename);
 		return -1;
 	      }
-	  if (g_strcasecmp (scanner->value.v_identifier, "true")==0)
+	  if (g_ascii_strcasecmp (scanner->value.v_identifier, "true")==0)
 	    extsync = 1;
-	  else if (g_strcasecmp (scanner->value.v_identifier, "false")==0)
+	  else if (g_ascii_strcasecmp (scanner->value.v_identifier, "false")==0)
 	    extsync = 0;
 	  else
 	    {
@@ -241,9 +243,9 @@ fb_modes_parse_mode (GScanner *scanner,
 		g_free (modename);
 		return -1;
 	      }
-	  if (g_strcasecmp (scanner->value.v_identifier, "true")==0)
+	  if (g_ascii_strcasecmp (scanner->value.v_identifier, "true")==0)
 	    doublescan = 1;
-	  else if (g_strcasecmp (scanner->value.v_identifier, "false")==0)
+	  else if (g_ascii_strcasecmp (scanner->value.v_identifier, "false")==0)
 	    doublescan = 0;
 	  else
 	    {
@@ -258,9 +260,9 @@ fb_modes_parse_mode (GScanner *scanner,
 		g_free (modename);
 		return -1;
 	      }
-	  if (g_strcasecmp (scanner->value.v_identifier, "high")==0)
+	  if (g_ascii_strcasecmp (scanner->value.v_identifier, "high")==0)
 	    vsync = 1;
-	  else if (g_strcasecmp (scanner->value.v_identifier, "low")==0)
+	  else if (g_ascii_strcasecmp (scanner->value.v_identifier, "low")==0)
 	    vsync = 0;
 	  else
 	    {
@@ -275,9 +277,9 @@ fb_modes_parse_mode (GScanner *scanner,
 	      g_free (modename);
 	      return -1;
 	    }
-	  if (g_strcasecmp (scanner->value.v_identifier, "high")==0)
+	  if (g_ascii_strcasecmp (scanner->value.v_identifier, "high")==0)
 	    hsync = 1;
-	  else if (g_strcasecmp (scanner->value.v_identifier, "low")==0)
+	  else if (g_ascii_strcasecmp (scanner->value.v_identifier, "low")==0)
 	    hsync = 0;
 	  else
 	    {
@@ -292,9 +294,9 @@ fb_modes_parse_mode (GScanner *scanner,
 	      g_free (modename);
 	      return -1;
 	    }
-	  if (g_strcasecmp (scanner->value.v_identifier, "high")==0)
+	  if (g_ascii_strcasecmp (scanner->value.v_identifier, "high")==0)
 	    csync = 1;
-	  else if (g_strcasecmp (scanner->value.v_identifier, "low")==0)
+	  else if (g_ascii_strcasecmp (scanner->value.v_identifier, "low")==0)
 	    csync = 0;
 	  else
 	    {
@@ -302,6 +304,23 @@ fb_modes_parse_mode (GScanner *scanner,
 	      return -1;
 	    }
 	  break;
+	case FB_ACCEL:
+	  token = g_scanner_get_next_token (scanner);
+	  if (token != G_TOKEN_IDENTIFIER)
+	    {
+	      g_free (modename);
+	      return -1;
+	    }
+	  if (g_ascii_strcasecmp (scanner->value.v_identifier, "false")==0)
+	    accel = 0;
+	  else if (g_ascii_strcasecmp (scanner->value.v_identifier, "true")==0)
+	    accel = 1;
+	  else
+	    {
+	      g_free (modename);
+	      return -1;
+	    }
+	  break;
 	}
       
       token = g_scanner_get_next_token (scanner);
@@ -314,6 +333,13 @@ fb_modes_parse_mode (GScanner *scanner,
     }
 
   if (strcmp (modename, specified_modename)== 0) {
+    /* we really should be parsing for rgba. regardless, if rgba isn't found,
+     * we can't assume that the original colors are valid for the new mode */
+    memset (&modeinfo->red, 0, sizeof (modeinfo->red));
+    memset (&modeinfo->green, 0, sizeof (modeinfo->green));
+    memset (&modeinfo->blue, 0, sizeof (modeinfo->blue));
+    memset (&modeinfo->transp, 0, sizeof (modeinfo->transp));
+
     if (!found_geometry)
       g_warning ("Geometry not specified");
 
@@ -350,6 +376,10 @@ fb_modes_parse_mode (GScanner *scanner,
 	  modeinfo->sync |= FB_SYNC_HOR_HIGH_ACT;
 	if (vsync)
 	  modeinfo->sync |= FB_SYNC_VERT_HIGH_ACT;
+	if (accel)
+	  modeinfo->accel_flags = FB_ACCELF_TEXT;
+	else
+	  modeinfo->accel_flags = 0;
       }
 
     g_free (modename);
@@ -386,7 +416,7 @@ gdk_fb_setup_mode_from_name (struct fb_var_screeninfo *modeinfo,
   scanner->input_name = filename;
 
   for (i = 0; i < sizeof(fb_modes_keywords)/sizeof(fb_modes_keywords[0]); i++)
-    g_scanner_add_symbol (scanner, fb_modes_keywords[i], GINT_TO_POINTER (i));
+    g_scanner_scope_add_symbol (scanner, 0, fb_modes_keywords[i], GINT_TO_POINTER (i));
 
   g_scanner_input_file (scanner, fd);
   
@@ -480,6 +510,13 @@ gdk_fb_set_mode (GdkFBDisplay *display)
       return -1;
     }
   
+  /* ask for info back to make sure of what we got */
+  if (ioctl (display->fb_fd, FBIOGET_VSCREENINFO, &display->modeinfo) < 0)
+    {
+      g_warning ("Error getting var screen info");
+      return -1;
+    }
+  
   if (ioctl (display->fb_fd, FBIOGET_FSCREENINFO, &display->sinfo) < 0)
     {
       g_warning ("Error getting fixed screen info");
@@ -668,7 +705,7 @@ gdk_fb_display_new ()
   s = getenv("GDK_VT");
   if (s)
     {
-      if (g_strcasecmp ("new", s)==0)
+      if (g_ascii_strcasecmp ("new", s)==0)
 	{
 	  n = ioctl (display->console_fd, VT_OPENQRY, &vt);
 	  if (n < 0 || vt == -1)
@@ -799,11 +836,11 @@ gdk_fb_display_destroy (GdkFBDisplay *display)
   g_free (display);
 }
 
-gboolean
-_gdk_windowing_init_check (int argc, char **argv)
+void
+_gdk_windowing_init (int *argc, char ***argv)
 {
   if (gdk_initialized)
-    return TRUE;
+    return;
 
   /* Create new session and become session leader */
   setsid();
@@ -811,7 +848,7 @@ _gdk_windowing_init_check (int argc, char **argv)
   gdk_display = gdk_fb_display_new ();
 
   if (!gdk_display)
-    return FALSE;
+    return;
 
   gdk_shadow_fb_init ();
   
@@ -822,7 +859,7 @@ _gdk_windowing_init_check (int argc, char **argv)
       g_warning ("Failed to initialize keyboard");
       gdk_fb_display_destroy (gdk_display);
       gdk_display = NULL;
-      return FALSE;
+      return;
     }
   
   if (!gdk_fb_mouse_init (!gdk_display->manager_blocked))
@@ -831,15 +868,24 @@ _gdk_windowing_init_check (int argc, char **argv)
       gdk_fb_keyboard_close ();
       gdk_fb_display_destroy (gdk_display);
       gdk_display = NULL;
-      return FALSE;
+      return;
     }
 
+  /* Although atexit is evil, we need it here because otherwise the
+   * keyboard is left in a bad state. you can still run 'reset' but
+   * that gets annoying after running testgtk for the 20th time.
+   */
+  g_atexit(_gdk_windowing_exit);
+
   gdk_initialized = TRUE;
 
   _gdk_selection_property = gdk_atom_intern ("GDK_SELECTION", FALSE);
 
-  
-  return TRUE;
+}
+
+void
+_gdk_windowing_set_default_display (GdkDisplay *display)
+{
 }
 
 /*
@@ -933,7 +979,7 @@ gdk_fb_pointer_grab (GdkWindow *	  window,
 
 /*
  *--------------------------------------------------------------
- * gdk_pointer_ungrab
+ * gdk_display_pointer_ungrab
  *
  *   Releases any pointer grab
  *
@@ -947,7 +993,8 @@ gdk_fb_pointer_grab (GdkWindow *	  window,
  */
 
 void
-gdk_pointer_ungrab (guint32 time)
+gdk_display_pointer_ungrab (GdkDisplay *display,
+			    guint32     time)
 {
   gdk_fb_pointer_ungrab (time, FALSE);
 }
@@ -993,7 +1040,7 @@ gdk_fb_pointer_ungrab (guint32 time, gboolean implicit_grab)
 
 /*
  *--------------------------------------------------------------
- * gdk_pointer_is_grabbed
+ * gdk_display_pointer_is_grabbed
  *
  *   Tell wether there is an active x pointer grab in effect
  *
@@ -1007,7 +1054,7 @@ gdk_fb_pointer_ungrab (guint32 time, gboolean implicit_grab)
  */
 
 gint
-gdk_pointer_is_grabbed (void)
+gdk_display_pointer_is_grabbed (GdkDisplay *display)
 {
   return _gdk_fb_pointer_grab_window != NULL;
 }
@@ -1033,9 +1080,9 @@ gdk_pointer_is_grabbed (void)
  */
 
 GdkGrabStatus
-gdk_keyboard_grab (GdkWindow *	   window,
-		   gint		   owner_events,
-		   guint32	   time)
+gdk_keyboard_grab (GdkWindow  *window,
+		   gint        owner_events,
+		   guint32     time)
 {
   g_return_val_if_fail (window != NULL, 0);
   g_return_val_if_fail (GDK_IS_WINDOW (window), 0);
@@ -1051,7 +1098,7 @@ gdk_keyboard_grab (GdkWindow *	   window,
 
 /*
  *--------------------------------------------------------------
- * gdk_keyboard_ungrab
+ * gdk_display_keyboard_ungrab
  *
  *   Releases any keyboard grab
  *
@@ -1065,7 +1112,8 @@ gdk_keyboard_grab (GdkWindow *	   window,
  */
 
 void
-gdk_keyboard_ungrab (guint32 time)
+gdk_display_keyboard_ungrab (GdkDisplay *display,
+			     guint32     time)
 {
   if (_gdk_fb_keyboard_grab_window)
     gdk_window_unref (_gdk_fb_keyboard_grab_window);
@@ -1073,7 +1121,8 @@ gdk_keyboard_ungrab (guint32 time)
 }
 
 gboolean
-gdk_pointer_grab_info_libgtk_only (GdkWindow **grab_window,
+gdk_pointer_grab_info_libgtk_only (GdkDisplay *display,
+				   GdkWindow **grab_window,
 				   gboolean   *owner_events)
 {
   if (_gdk_fb_pointer_grab_window)
@@ -1090,8 +1139,9 @@ gdk_pointer_grab_info_libgtk_only (GdkWindow **grab_window,
 }
 
 gboolean
-gdk_keyboard_grab_info_libgtk_only (GdkWindow **grab_window,
-                                   gboolean   *owner_events)
+gdk_keyboard_grab_info_libgtk_only (GdkDisplay *display,
+				    GdkWindow **grab_window,
+				    gboolean   *owner_events)
 {
   if (_gdk_fb_keyboard_grab_window)
     {
@@ -1109,7 +1159,7 @@ gdk_keyboard_grab_info_libgtk_only (GdkWindow **grab_window,
 
 /*
  *--------------------------------------------------------------
- * gdk_screen_width
+ * gdk_screen_get_width
  *
  *   Return the width of the screen.
  *
@@ -1123,14 +1173,14 @@ gdk_keyboard_grab_info_libgtk_only (GdkWindow **grab_window,
  */
 
 gint
-gdk_screen_width (void)
+gdk_screen_get_width (GdkScreen *screen)
 {
   return gdk_display->fb_width;
 }
 
 /*
  *--------------------------------------------------------------
- * gdk_screen_height
+ * gdk_screen_get_height
  *
  *   Return the height of the screen.
  *
@@ -1144,14 +1194,14 @@ gdk_screen_width (void)
  */
 
 gint
-gdk_screen_height (void)
+gdk_screen_get_height (GdkScreen *screen)
 {
   return gdk_display->fb_height;
 }
 
 /*
  *--------------------------------------------------------------
- * gdk_screen_width_mm
+ * gdk_screen_get_width_mm
  *
  *   Return the width of the screen in millimeters.
  *
@@ -1165,14 +1215,14 @@ gdk_screen_height (void)
  */
 
 gint
-gdk_screen_width_mm (void)
+gdk_screen_get_width_mm (GdkScreen *screen)
 {
   return 0.5 + gdk_screen_width () * (25.4 / 72.);
 }
 
 /*
  *--------------------------------------------------------------
- * gdk_screen_height
+ * gdk_screen_get_height_mm
  *
  *   Return the height of the screen in millimeters.
  *
@@ -1186,39 +1236,17 @@ gdk_screen_width_mm (void)
  */
 
 gint
-gdk_screen_height_mm (void)
+gdk_screen_get_height_mm (GdkScreen *screen)
 {
   return 0.5 + gdk_screen_height () * (25.4 / 72.);
 }
 
-/*
- *--------------------------------------------------------------
- * gdk_set_sm_client_id
- *
- *   Set the SM_CLIENT_ID property on the WM_CLIENT_LEADER window
- *   so that the window manager can save our state using the
- *   X11R6 ICCCM session management protocol. A NULL value should 
- *   be set following disconnection from the session manager to
- *   remove the SM_CLIENT_ID property.
- *
- * Arguments:
- * 
- *   "sm_client_id" specifies the client id assigned to us by the
- *   session manager or NULL to remove the property.
- *
- * Results:
- *
- * Side effects:
- *
- *--------------------------------------------------------------
- */
-
 void
-gdk_set_sm_client_id (const gchar* sm_client_id)
+_gdk_windowing_display_set_sm_client_id (GdkDisplay*  display,
+					 const gchar* sm_client_id)
 {
 }
 
-
 extern void keyboard_shutdown(void);
 
 void
@@ -1236,18 +1264,6 @@ _gdk_windowing_exit (void)
   gdk_display = NULL;
 }
 
-gchar*
-gdk_keyval_name (guint	      keyval)
-{
-  return NULL;
-}
-
-guint
-gdk_keyval_from_name (const gchar *keyval_name)
-{
-  return 0;
-}
-
 gchar *
 gdk_get_display(void)
 {
@@ -1260,9 +1276,8 @@ gdk_get_display(void)
   return g_strdup (s);
 }
 
-
 void
-gdk_beep (void)
+gdk_display_beep (GdkDisplay *display)
 {
   static int pitch = 600, duration = 100;
   gulong arg;
@@ -1326,7 +1341,7 @@ gdk_fb_other_event_window (GdkWindow *window,
 	  (g_object_get_data (G_OBJECT (w), "gdk-window-child-handler")))
 	  break;
 	  
-      evmask = GDK_WINDOW_IMPL_FBDATA(window)->event_mask;
+      evmask = GDK_WINDOW_OBJECT(window)->event_mask;
 
       if (evmask & type_masks[type])
 	return w;
@@ -1375,7 +1390,7 @@ gdk_fb_pointer_event_window (GdkWindow *window,
 	  (g_object_get_data (G_OBJECT (w), "gdk-window-child-handler")))
 	  break;
       
-      evmask = GDK_WINDOW_IMPL_FBDATA(window)->event_mask;
+      evmask = GDK_WINDOW_OBJECT(window)->event_mask;
 
       if (evmask & (GDK_BUTTON1_MOTION_MASK | GDK_BUTTON2_MOTION_MASK | GDK_BUTTON3_MOTION_MASK))
 	{
@@ -1433,7 +1448,7 @@ gdk_fb_keyboard_event_window (GdkWindow *window,
 	  (g_object_get_data (G_OBJECT (w), "gdk-window-child-handler")))
 	  break;
 	  
-      evmask = GDK_WINDOW_IMPL_FBDATA(window)->event_mask;
+      evmask = GDK_WINDOW_OBJECT(window)->event_mask;
 
       if (evmask & type_masks[type])
 	return w;
@@ -1455,12 +1470,11 @@ gdk_event_make (GdkWindow *window,
 		GdkEventType type,
 		gboolean append_to_queue)
 {
-  GdkEvent *event = _gdk_event_new ();
+  GdkEvent *event = gdk_event_new (type);
   guint32 the_time;
   
   the_time = gdk_fb_get_time ();
   
-  event->any.type = type;
   event->any.window = gdk_window_ref (window);
   event->any.send_event = FALSE;
   switch (type)
@@ -1523,7 +1537,7 @@ gdk_event_make (GdkWindow *window,
     }
   
   if (append_to_queue)
-    _gdk_event_queue_append (event);
+    _gdk_event_queue_append (gdk_display_get_default (), event);
   
   return event;
 }
@@ -1578,3 +1592,8 @@ gdk_error_trap_pop (void)
 {
   return 0;
 }
+
+void
+gdk_notify_startup_complete (void)
+{
+}
