@@ -30,7 +30,6 @@
 #include "gdkprivate-x11.h"
 #include "gdkregion-generic.h"
 #include "gdkx.h"
-#include "gdkalias.h"
 
 #include <string.h>
 
@@ -54,16 +53,47 @@ static void gdk_x11_gc_set_dashes (GdkGC           *gc,
 				   gint8            dash_list[],
 				   gint             n);
 
-static void gdk_gc_x11_finalize   (GObject         *object);
+static void gdk_gc_x11_class_init (GdkGCX11Class *klass);
+static void gdk_gc_x11_finalize   (GObject           *object);
 
-G_DEFINE_TYPE (GdkGCX11, _gdk_gc_x11, GDK_TYPE_GC)
+static gpointer parent_class = NULL;
+
+GType
+_gdk_gc_x11_get_type (void)
+{
+  static GType object_type = 0;
+
+  if (!object_type)
+    {
+      static const GTypeInfo object_info =
+      {
+        sizeof (GdkGCX11Class),
+        (GBaseInitFunc) NULL,
+        (GBaseFinalizeFunc) NULL,
+        (GClassInitFunc) gdk_gc_x11_class_init,
+        NULL,           /* class_finalize */
+        NULL,           /* class_data */
+        sizeof (GdkGCX11),
+        0,              /* n_preallocs */
+        (GInstanceInitFunc) NULL,
+      };
+      
+      object_type = g_type_register_static (GDK_TYPE_GC,
+                                            "GdkGCX11",
+                                            &object_info, 0);
+    }
+  
+  return object_type;
+}
 
 static void
-_gdk_gc_x11_class_init (GdkGCX11Class *klass)
+gdk_gc_x11_class_init (GdkGCX11Class *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
   GdkGCClass *gc_class = GDK_GC_CLASS (klass);
   
+  parent_class = g_type_class_peek_parent (klass);
+
   object_class->finalize = gdk_gc_x11_finalize;
 
   gc_class->get_values = gdk_x11_gc_get_values;
@@ -71,19 +101,22 @@ _gdk_gc_x11_class_init (GdkGCX11Class *klass)
   gc_class->set_dashes = gdk_x11_gc_set_dashes;
 }
 
-static void
-_gdk_gc_x11_init (GdkGCX11 *gc)
-{
-}
-
 static void
 gdk_gc_x11_finalize (GObject *object)
 {
   GdkGCX11 *x11_gc = GDK_GC_X11 (object);
   
+  if (x11_gc->clip_region)
+    gdk_region_destroy (x11_gc->clip_region);
+  
+#if HAVE_XFT
+  if (x11_gc->fg_picture != None)
+    XRenderFreePicture (GDK_GC_XDISPLAY (x11_gc), x11_gc->fg_picture);
+#endif  
+  
   XFreeGC (GDK_GC_XDISPLAY (x11_gc), GDK_GC_XGC (x11_gc));
 
-  G_OBJECT_CLASS (_gdk_gc_x11_parent_class)->finalize (object);
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 
@@ -106,15 +139,11 @@ _gdk_x11_gc_new (GdkDrawable      *drawable,
   gc = g_object_new (_gdk_gc_x11_get_type (), NULL);
   private = GDK_GC_X11 (gc);
 
-  _gdk_gc_init (gc, drawable, values, values_mask);
-
   private->dirty_mask = 0;
-  private->have_clip_mask = FALSE;
+  private->clip_region = NULL;
     
   private->screen = GDK_DRAWABLE_IMPL_X11 (drawable)->screen;
 
-  private->depth = gdk_drawable_get_depth (drawable);
-
   if (values_mask & (GDK_GC_CLIP_X_ORIGIN | GDK_GC_CLIP_Y_ORIGIN))
     {
       values_mask &= ~(GDK_GC_CLIP_X_ORIGIN | GDK_GC_CLIP_Y_ORIGIN);
@@ -127,9 +156,9 @@ _gdk_x11_gc_new (GdkDrawable      *drawable,
       private->dirty_mask |= GDK_GC_DIRTY_TS;
     }
 
-  if ((values_mask & GDK_GC_CLIP_MASK) && values->clip_mask)
-    private->have_clip_mask = TRUE;
-
+  if (values_mask & GDK_GC_FOREGROUND)
+    private->fg_pixel = values->foreground.pixel;
+  
   xvalues.function = GXcopy;
   xvalues.fill_style = FillSolid;
   xvalues.arc_mode = ArcPieSlice;
@@ -149,29 +178,25 @@ _gdk_x11_gc_new (GdkDrawable      *drawable,
 GC
 _gdk_x11_gc_flush (GdkGC *gc)
 {
-  Display *xdisplay = GDK_GC_XDISPLAY (gc);
   GdkGCX11 *private = GDK_GC_X11 (gc);
-  GC xgc = private->xgc;
 
   if (private->dirty_mask & GDK_GC_DIRTY_CLIP)
     {
-      GdkRegion *clip_region = _gdk_gc_get_clip_region (gc);
-      
-      if (!clip_region)
-	XSetClipOrigin (xdisplay, xgc,
+      if (!private->clip_region)
+	XSetClipOrigin (GDK_GC_XDISPLAY (gc), GDK_GC_XGC (gc),
 			gc->clip_x_origin, gc->clip_y_origin);
       else
 	{
 	  XRectangle *rectangles;
           gint n_rects;
 
-          _gdk_region_get_xrectangles (clip_region,
+          _gdk_region_get_xrectangles (private->clip_region,
                                        gc->clip_x_origin,
                                        gc->clip_y_origin,
                                        &rectangles,
                                        &n_rects);
 	  
-	  XSetClipRectangles (xdisplay, xgc, 0, 0,
+	  XSetClipRectangles (GDK_GC_XDISPLAY (gc), GDK_GC_XGC (gc), 0, 0,
                               rectangles,
                               n_rects, YXBanded);
           
@@ -181,12 +206,12 @@ _gdk_x11_gc_flush (GdkGC *gc)
 
   if (private->dirty_mask & GDK_GC_DIRTY_TS)
     {
-      XSetTSOrigin (xdisplay, xgc,
+      XSetTSOrigin (GDK_GC_XDISPLAY (gc), GDK_GC_XGC (gc),
 		    gc->ts_x_origin, gc->ts_y_origin);
     }
 
   private->dirty_mask = 0;
-  return xgc;
+  return GDK_GC_XGC (gc);
 }
 
 static void
@@ -338,6 +363,7 @@ gdk_x11_gc_get_values (GdkGC       *gc,
     }
 }
 
+
 static void
 gdk_x11_gc_set_values (GdkGC           *gc,
 		       GdkGCValues     *values,
@@ -347,6 +373,8 @@ gdk_x11_gc_set_values (GdkGC           *gc,
   XGCValues xvalues;
   unsigned long xvalues_mask = 0;
 
+  g_return_if_fail (GDK_IS_GC (gc));
+
   x11_gc = GDK_GC_X11 (gc);
 
   if (values_mask & (GDK_GC_CLIP_X_ORIGIN | GDK_GC_CLIP_Y_ORIGIN))
@@ -363,10 +391,16 @@ gdk_x11_gc_set_values (GdkGC           *gc,
 
   if (values_mask & GDK_GC_CLIP_MASK)
     {
-      x11_gc->have_clip_region = FALSE;
-      x11_gc->have_clip_mask = values->clip_mask != NULL;
+      if (x11_gc->clip_region)
+	{
+	  gdk_region_destroy (x11_gc->clip_region);
+	  x11_gc->clip_region = NULL;
+	}
     }
 
+  if (values_mask & GDK_GC_FOREGROUND)
+    x11_gc->fg_pixel = values->foreground.pixel;
+  
   gdk_x11_gc_values_to_xvalues (values, values_mask, &xvalues, &xvalues_mask);
 
   XChangeGC (GDK_GC_XDISPLAY (gc),
@@ -385,7 +419,7 @@ gdk_x11_gc_set_dashes (GdkGC *gc,
   g_return_if_fail (dash_list != NULL);
 
   XSetDashes (GDK_GC_XDISPLAY (gc), GDK_GC_XGC (gc),
-	      dash_offset, (char *)dash_list, n);
+	      dash_offset, dash_list, n);
 }
 
 static void
@@ -607,41 +641,98 @@ gdk_x11_gc_values_to_xvalues (GdkGCValues    *values,
 }
 
 void
-_gdk_windowing_gc_set_clip_region (GdkGC     *gc,
-				   GdkRegion *region)
+gdk_gc_set_clip_rectangle (GdkGC	*gc,
+			   GdkRectangle *rectangle)
 {
-  GdkGCX11 *x11_gc = GDK_GC_X11 (gc);
+  GdkGCX11 *x11_gc;
 
-  /* Unset immediately, to make sure Xlib doesn't keep the
-   * XID of an old clip mask cached
-   */
-  if ((x11_gc->have_clip_region && !region) || x11_gc->have_clip_mask)
+  g_return_if_fail (GDK_IS_GC (gc));
+
+  x11_gc = GDK_GC_X11 (gc);
+
+  if (x11_gc->clip_region)
+    gdk_region_destroy (x11_gc->clip_region);
+
+  if (rectangle)
+    x11_gc->clip_region = gdk_region_rectangle (rectangle);
+  else
     {
+      x11_gc->clip_region = NULL;
       XSetClipMask (GDK_GC_XDISPLAY (gc), GDK_GC_XGC (gc), None);
-      x11_gc->have_clip_mask = FALSE;
     }
 
-  x11_gc->have_clip_region = region != NULL;
-
   gc->clip_x_origin = 0;
   gc->clip_y_origin = 0;
-
+  
   x11_gc->dirty_mask |= GDK_GC_DIRTY_CLIP;
 }
 
 void
-_gdk_windowing_gc_copy (GdkGC *dst_gc,
-			GdkGC *src_gc)
+gdk_gc_set_clip_region (GdkGC	  *gc,
+			GdkRegion *region)
 {
-  GdkGCX11 *x11_src_gc = GDK_GC_X11 (src_gc);
-  GdkGCX11 *x11_dst_gc = GDK_GC_X11 (dst_gc);
+  GdkGCX11 *x11_gc;
 
+  g_return_if_fail (GDK_IS_GC (gc));
+
+  x11_gc = GDK_GC_X11 (gc);
+
+  if (x11_gc->clip_region)
+    gdk_region_destroy (x11_gc->clip_region);
+
+  if (region)
+    x11_gc->clip_region = gdk_region_copy (region);
+  else
+    {
+      x11_gc->clip_region = NULL;
+      XSetClipMask (GDK_GC_XDISPLAY (gc), GDK_GC_XGC (gc), None);
+    }
+  
+  gc->clip_x_origin = 0;
+  gc->clip_y_origin = 0;
+  
+  x11_gc->dirty_mask |= GDK_GC_DIRTY_CLIP;
+}
+
+
+void
+gdk_gc_copy (GdkGC *dst_gc, GdkGC *src_gc)
+{
+  GdkGCX11 *x11_src_gc;
+  GdkGCX11 *x11_dst_gc;
+  
+  g_return_if_fail (GDK_IS_GC_X11 (dst_gc));
+  g_return_if_fail (GDK_IS_GC_X11 (src_gc));
+
+  x11_dst_gc = GDK_GC_X11 (dst_gc);
+  x11_src_gc = GDK_GC_X11 (src_gc);
+  
   XCopyGC (GDK_GC_XDISPLAY (src_gc), GDK_GC_XGC (src_gc), ~((~1) << GCLastBit),
 	   GDK_GC_XGC (dst_gc));
 
+  dst_gc->clip_x_origin = src_gc->clip_x_origin;
+  dst_gc->clip_y_origin = src_gc->clip_y_origin;
+  dst_gc->ts_x_origin = src_gc->ts_x_origin;
+  dst_gc->ts_y_origin = src_gc->ts_y_origin;
+
+  if (src_gc->colormap)
+    g_object_ref (src_gc->colormap);
+
+  if (dst_gc->colormap)
+    g_object_unref (dst_gc->colormap);
+
+  dst_gc->colormap = src_gc->colormap;
+
+  if (x11_dst_gc->clip_region)
+    gdk_region_destroy (x11_dst_gc->clip_region);
+
+  if (x11_src_gc->clip_region)
+    x11_dst_gc->clip_region = gdk_region_copy (x11_src_gc->clip_region);
+  else
+    x11_dst_gc->clip_region = NULL;
+
   x11_dst_gc->dirty_mask = x11_src_gc->dirty_mask;
-  x11_dst_gc->have_clip_region = x11_src_gc->have_clip_region;
-  x11_dst_gc->have_clip_mask = x11_src_gc->have_clip_mask;
+  x11_dst_gc->fg_pixel = x11_src_gc->fg_pixel;
 }
 
 /**
@@ -651,8 +742,6 @@ _gdk_windowing_gc_copy (GdkGC *dst_gc,
  * Gets the #GdkScreen for which @gc was created
  *
  * Returns: the #GdkScreen for @gc.
- *
- * Since: 2.2
  */
 GdkScreen *  
 gdk_gc_get_screen (GdkGC *gc)
@@ -701,5 +790,131 @@ gdk_x11_gc_get_xgc (GdkGC *gc)
   return gc_x11->xgc;
 }
 
-#define __GDK_GC_X11_C__
-#include "gdkaliasdef.c"
+#ifdef HAVE_XFT
+/* Various bits of the below are roughly cribbed from XFree86
+ * lib/Xft/xftdraw.c, Copyright 2000, Keith Packard
+ */
+
+static XRenderPictFormat *
+foreground_format (GdkGC *gc)
+{
+  XRenderPictFormat pf;
+
+  pf.type = PictTypeDirect;
+  pf.depth = 32;
+  pf.direct.redMask = 0xff;
+  pf.direct.greenMask = 0xff;
+  pf.direct.blueMask = 0xff;
+  pf.direct.alphaMask = 0xff;
+
+  return XRenderFindFormat (GDK_GC_XDISPLAY (gc),
+			    (PictFormatType |
+			     PictFormatDepth |
+			     PictFormatRedMask |
+			     PictFormatGreenMask |
+			     PictFormatBlueMask |
+			     PictFormatAlphaMask),
+			    &pf,
+			    0);
+}
+
+/**
+ * _gdk_x11_gc_get_fg_picture:
+ * @gc: a #GdkGC
+ * 
+ * Gets a Xrender Picture object suitable for being the source
+ * drawable for drawing with the foreground the graphics context.
+ * (Currently, only foreground color is handled, but in the
+ * future we should handle tiles/stipples as well.)
+ * 
+ * Return value: a Picture, owned by the GC; this cannot be
+ *   used over subsequent modification of the GC.
+ **/
+Picture
+_gdk_x11_gc_get_fg_picture (GdkGC *gc)
+{
+  GdkGCX11 *x11_gc;
+  GdkColormap *cmap;
+  gboolean new = FALSE;
+  GdkColor color;
+  
+  g_return_val_if_fail (GDK_IS_GC_X11 (gc), None);
+
+  if (!_gdk_x11_have_render (GDK_GC_DISPLAY (gc)))
+    return None;
+
+  x11_gc = GDK_GC_X11 (gc);
+  cmap = gdk_gc_get_colormap (gc);
+
+  if (x11_gc->fg_picture == None)
+    {
+      XRenderPictureAttributes pa;
+      XRenderPictFormat *pix_format = foreground_format (gc);
+      Pixmap pix;
+
+      if (!pix_format)
+	return None;
+
+      pix = XCreatePixmap (GDK_GC_XDISPLAY (gc), 
+			   GDK_SCREEN_XROOTWIN (x11_gc->screen),
+			   1, 1, pix_format->depth);
+      pa.repeat = True;
+      x11_gc->fg_picture = XRenderCreatePicture (GDK_GC_XDISPLAY (gc), 
+						 pix,
+						 pix_format,
+						 CPRepeat, &pa);
+      XFreePixmap (GDK_GC_XDISPLAY (gc), pix);
+      
+      new = TRUE;
+    }
+
+  gdk_colormap_query_color (cmap, x11_gc->fg_pixel, &color);
+
+  if (new ||
+      x11_gc->fg_picture_color.red != color.red ||
+      x11_gc->fg_picture_color.green != color.green ||
+      x11_gc->fg_picture_color.blue != color.blue)
+    {
+      x11_gc->fg_picture_color.red = color.red;
+      x11_gc->fg_picture_color.green = color.green;
+      x11_gc->fg_picture_color.blue = color.blue;
+      x11_gc->fg_picture_color.alpha = 0xffff;
+
+      XRenderFillRectangle (GDK_GC_XDISPLAY (gc), PictOpSrc, 
+			    x11_gc->fg_picture, &x11_gc->fg_picture_color,
+			    0, 0, 1, 1); 
+    }
+
+  return x11_gc->fg_picture;
+}
+
+/**
+ * _gdk_gc_x11_get_fg_xft_color:
+ * @gc: a #GdkGC
+ * @xftcolor: location to store the color
+ * 
+ * Gets the foreground color of the GC as a XftColor.
+ **/
+void
+_gdk_gc_x11_get_fg_xft_color (GdkGC    *gc,
+			      XftColor *xftcolor)
+{
+  GdkGCX11 *x11_gc;
+  GdkColormap *cmap;
+  GdkColor color;
+  
+  g_return_if_fail (GDK_IS_GC_X11 (gc));
+
+  x11_gc = GDK_GC_X11 (gc);
+  cmap = gdk_gc_get_colormap (gc);
+
+  xftcolor->pixel = x11_gc->fg_pixel;
+
+  gdk_colormap_query_color (cmap, xftcolor->pixel, &color);
+  xftcolor->color.red = color.red;
+  xftcolor->color.green = color.green;
+  xftcolor->color.blue = color.blue;
+  xftcolor->color.alpha = 0xffff;
+}
+
+#endif /* HAVE_XFT */
