@@ -22,6 +22,111 @@
  *
  * By Owen Taylor <otaylor@redhat.com>
  * Copyright Red Hat, Inc. 2000
+ *
+ * The algorithms implemented in this file are an extension of the
+ * idea of guffaw scrolling, a technique (and name) taken from the classic
+ * Netscape source code. The basic idea of guffaw scrolling is a trick
+ * to get around a limitation of X: there is no way of scrolling the
+ * contents of a window. Guffaw scrolling exploits the X concepts of
+ * window gravity and bit gravity:
+ *
+ *  window gravity: the window gravity of a window affects what happens
+ *   to a windows position when _its parent_ is resized, or
+ *   moved and resized simultaneously.
+ *
+ *  bit gravity: the bit gravity of a window affects what happens to
+ *  the pixels of a window when _it_ is is resized, or moved and
+ *  resized simultaneously.
+ *
+ * These were basically intended to do things like have right
+ * justified widgets in a window automatically stay right justified
+ * when the window was resized, but there is also the special
+ * "StaticGravity" which means "do nothing." We can exploit
+ * StaticGravity to scroll a window:
+ *
+ *     |  VISIBLE  |
+ * 
+ *     |abcdefghijk|
+ *     |abcdefghijk    |   (1) Resize bigger
+ * |    efghijk    |       (2) Move  
+ *     |efghijk    |       (3) Move-resize back to the original size
+ *
+ * Or, going the other way:
+
+ *     |abcdefghijk|
+ * |    abcdefghijk|       (1) Move-resize bigger
+ *     |    abcdefghijk|   (2) Move  
+ *     |    abcdefg|       (4) Resize back to the original size
+ *
+ * By using this technique, we can simulate scrolling around in a
+ * large virtual space without having to actually have windows that
+ * big; for the pixels of the window, this is all we have to do.  For
+ * subwindows, we have to take care of one other detail - since
+ * coordinates in X are limited to 16 bits, subwindows scrolled off
+ * will wrap around and come back eventually. So, we have to take care
+ * to unmap windows that go outside the 16-bit range and remap them as
+ * they come back in.
+ *
+ * Since we are temporarily making the window bigger, this only looks
+ * good if the edges of the window are obscured. Typically, we do
+ * this by making the window we are scrolling the immediate child
+ * of a "clip window".
+ *
+ * But, this isn't a perfect API for applications for several reasons:
+ *
+ *  - We have to use this inefficient technique even for small windows
+ *    if the window _could_ be big.
+ *  - Applications have to use a special scrolling API.
+ *
+ * What we'd like is to simply have windows with 32 bit coordinates
+ * so applications could scroll in the classic way - just move a big
+ * window around.
+ *
+ * It turns out that StaticGravity can also be used to achieve emulation
+ * of 32 bit coordinates with only 16 bit coordinates if we expand
+ * our horizons just a bit; what guffaw scrolling really is is a way
+ * to move the contents of a window a different amount than we move
+ * the borders of of the window. In the above example pictures we
+ * ended up with the borders of the window not moving at all, but
+ * that isn't necessary.
+ *
+ * So, what we do is set up a mapping from virtual 32 bit window position/size
+ * to:
+ *
+ *  - Real window position/size
+ *  - Offset between virtual coordinates and real coordinates for the window
+ *  - Map state (mapped or unmapped)
+ *
+ * By the following rules:
+ *
+ *  - If the window is less than 32767 pixels in width (resp. height), we use it's
+ *    virtual width and position.
+ *  - Otherwise, we use a width of 32767 and determine the position of the window
+ *    so that the portion of the real window [16384, 16383] in _toplevel window
+ *    coordinates_ is the same as the portion of the real window 
+ *
+ * This is implemented in gdk_window_compute_position(). Then the algorithm
+ * for a moving a window (_window_move_resize_child ()) is:
+ * 
+ *  - Compute the new window mappings for the window and all subwindows
+ *  - Expand out the boundary of the window and all subwindows by the amount
+ *    that the real/virtual offset changes for each window. 
+ *    (compute_intermediate_position() computes expanded boundary)
+ *  - Move the toplevel by the amount that it's contents need to translate.
+ *  - Move/resize the window and all subwindows to the newly computed
+ *    positions.
+ *
+ * If we just are scrolling (gdk_window_guffaw_scroll()), then things
+ * are similar, except that the final mappings for the toplevel are
+ * the same as the initial mappings, but we act as if it moved by the
+ * amount we are scrolling by.
+ *
+ * Note that we don't have to worry about a clip window in
+ * _gdk_window_move_resize() since we have set up our translation so
+ * that things in the range [16384,16383] in toplevel window
+ * coordinates look exactly as they would if we were simply moving the
+ * windows, and nothing outside this range is going to be visible
+ * unless the user has a _really_ huge screen.
  */
 
 #include "gdk.h"		/* For gdk_rectangle_intersect */
@@ -29,6 +134,8 @@
 #include "gdkx.h"
 #include "gdkregion.h"
 #include "gdkinternals.h"
+#include "gdkscreen-x11.h"
+#include "gdkdisplay-x11.h"
 
 typedef struct _GdkWindowQueueItem GdkWindowQueueItem;
 typedef struct _GdkWindowParentPos GdkWindowParentPos;
@@ -81,8 +188,6 @@ static void gdk_window_clip_changed       (GdkWindow          *window,
 					   GdkRectangle       *old_clip,
 					   GdkRectangle       *new_clip);
 
-static GSList *translate_queue = NULL;
-
 void
 _gdk_windowing_window_get_offsets (GdkWindow *window,
 				   gint      *x_offset,
@@ -110,6 +215,169 @@ _gdk_window_init_position (GdkWindow *window)
   gdk_window_compute_position (impl, &parent_pos, &impl->position_info);
 }
 
+static void
+gdk_window_copy_area_scroll (GdkWindow    *window,
+			     GdkRectangle *dest_rect,
+			     gint          dx,
+			     gint          dy)
+{
+  GdkWindowObject *obj = GDK_WINDOW_OBJECT (window);
+  GList *tmp_list;
+
+  if (dest_rect->width > 0 && dest_rect->height > 0)
+    {
+      GC gc;
+      XGCValues values;
+
+      values.graphics_exposures = True;
+      gc = XCreateGC (GDK_WINDOW_XDISPLAY (window),
+		      GDK_WINDOW_XID (window),
+		      GCGraphicsExposures, &values);
+
+      gdk_window_queue_translation (window, dx, dy);
+
+      XCopyArea (GDK_WINDOW_XDISPLAY (window),
+		 GDK_WINDOW_XID (window),
+		 GDK_WINDOW_XID (window),
+		 gc,
+		 dest_rect->x - dx, dest_rect->y - dy,
+		 dest_rect->width, dest_rect->height,
+		 dest_rect->x, dest_rect->y);
+
+      XFreeGC (GDK_WINDOW_XDISPLAY (window), gc);
+    }
+
+  tmp_list = obj->children;
+  while (tmp_list)
+    {
+      GdkWindow *child = GDK_WINDOW (tmp_list->data);
+      GdkWindowObject *child_obj = GDK_WINDOW_OBJECT (child);
+	  
+      gdk_window_move (child, child_obj->x + dx, child_obj->y + dy);
+      
+      tmp_list = tmp_list->next;
+    }
+}
+
+static void
+compute_intermediate_position (GdkXPositionInfo *position_info,
+			       GdkXPositionInfo *new_info,
+			       gint              d_xoffset,
+			       gint              d_yoffset,
+			       GdkRectangle     *new_position)
+{
+  gint new_x0, new_x1, new_y0, new_y1;
+  
+  /* Wrap d_xoffset, d_yoffset into [-32768,32767] range. For the
+   * purposes of subwindow movement, it doesn't matter if we are
+   * off by a factor of 65536, and if we don't do this range
+   * reduction, we'll end up with invalid widths.
+   */
+  d_xoffset = (gint16)d_xoffset;
+  d_yoffset = (gint16)d_yoffset;
+  
+  if (d_xoffset < 0)
+    {
+      new_x0 = position_info->x + d_xoffset;
+      new_x1 = position_info->x + position_info->width;
+    }
+  else
+    {
+      new_x0 = position_info->x;
+      new_x1 = position_info->x + new_info->width + d_xoffset;
+    }
+
+  new_position->x = new_x0;
+  new_position->width = new_x1 - new_x0;
+  
+  if (d_yoffset < 0)
+    {
+      new_y0 = position_info->y + d_yoffset;
+      new_y1 = position_info->y + position_info->height;
+    }
+  else
+    {
+      new_y0 = position_info->y;
+      new_y1 = position_info->y + new_info->height + d_yoffset;
+    }
+  
+  new_position->y = new_y0;
+  new_position->height = new_y1 - new_y0;
+}
+
+static void
+gdk_window_guffaw_scroll (GdkWindow    *window,
+			  gint          dx,
+			  gint          dy)
+{
+  GdkWindowObject *obj = GDK_WINDOW_OBJECT (window);
+  GdkWindowImplX11 *impl = GDK_WINDOW_IMPL_X11 (obj->impl);
+
+  gint d_xoffset = -dx;
+  gint d_yoffset = -dy;
+  GdkRectangle new_position;
+  GdkXPositionInfo new_info;
+  GdkWindowParentPos parent_pos;
+  GList *tmp_list;
+  
+  gdk_window_compute_parent_pos (impl, &parent_pos);
+  gdk_window_compute_position (impl, &parent_pos, &new_info);
+
+  parent_pos.x += obj->x;
+  parent_pos.y += obj->y;
+  parent_pos.x11_x += new_info.x;
+  parent_pos.x11_y += new_info.y;
+  parent_pos.clip_rect = new_info.clip_rect;
+
+  gdk_window_tmp_unset_bg (window);
+
+  if (d_xoffset < 0 || d_yoffset < 0)
+    gdk_window_queue_translation (window, MIN (d_xoffset, 0), MIN (d_yoffset, 0));
+	
+  gdk_window_set_static_gravities (window, TRUE);
+
+  compute_intermediate_position (&impl->position_info, &new_info, d_xoffset, d_yoffset,
+				 &new_position);
+  
+  XMoveResizeWindow (GDK_WINDOW_XDISPLAY (window),
+		     GDK_WINDOW_XID (window),
+		     new_position.x, new_position.y, new_position.width, new_position.height);
+  
+  tmp_list = obj->children;
+  while (tmp_list)
+    {
+      GDK_WINDOW_OBJECT(tmp_list->data)->x -= d_xoffset;
+      GDK_WINDOW_OBJECT(tmp_list->data)->y -= d_yoffset;
+
+      gdk_window_premove (tmp_list->data, &parent_pos);
+      tmp_list = tmp_list->next;
+    }
+  
+  XMoveWindow (GDK_WINDOW_XDISPLAY (window),
+	       GDK_WINDOW_XID (window),
+	       new_position.x - d_xoffset, new_position.y - d_yoffset);
+  
+  if (d_xoffset > 0 || d_yoffset > 0)
+    gdk_window_queue_translation (window, MAX (d_xoffset, 0), MAX (d_yoffset, 0));
+  
+  XMoveResizeWindow (GDK_WINDOW_XDISPLAY (window),
+		     GDK_WINDOW_XID (window),
+		     impl->position_info.x, impl->position_info.y,
+		     impl->position_info.width, impl->position_info.height);
+  
+  if (impl->position_info.no_bg)
+    gdk_window_tmp_reset_bg (window);
+  
+  impl->position_info = new_info;
+  
+  tmp_list = obj->children;
+  while (tmp_list)
+    {
+      gdk_window_postmove (tmp_list->data, &parent_pos);
+      tmp_list = tmp_list->next;
+    }
+}
+
 /**
  * gdk_window_scroll:
  * @window: a #GdkWindow
@@ -131,97 +399,60 @@ gdk_window_scroll (GdkWindow *window,
 		   gint       dy)
 {
   gboolean can_guffaw_scroll = FALSE;
+  GdkRegion *invalidate_region;
   GdkWindowImplX11 *impl;
   GdkWindowObject *obj;
+  GdkRectangle dest_rect;
   
   g_return_if_fail (GDK_IS_WINDOW (window));
 
-  obj = GDK_WINDOW_OBJECT (window);
-  
-  impl = GDK_WINDOW_IMPL_X11 (obj->impl);  
-  
   if (GDK_WINDOW_DESTROYED (window))
     return;
   
+  obj = GDK_WINDOW_OBJECT (window);
+  impl = GDK_WINDOW_IMPL_X11 (obj->impl);  
+
+  if (dx == 0 && dy == 0)
+    return;
+  
   /* Move the current invalid region */
   if (obj->update_area)
     gdk_region_offset (obj->update_area, dx, dy);
   
-  /* We can guffaw scroll if we are a child window, and the parent
-   * does not extend beyond our edges.
-   */
+  invalidate_region = gdk_region_rectangle (&impl->position_info.clip_rect);
   
+  dest_rect = impl->position_info.clip_rect;
+  dest_rect.x += dx;
+  dest_rect.y += dy;
+  gdk_rectangle_intersect (&dest_rect, &impl->position_info.clip_rect, &dest_rect);
+
+  if (dest_rect.width > 0 && dest_rect.height > 0)
+    {
+      GdkRegion *tmp_region;
+
+      tmp_region = gdk_region_rectangle (&dest_rect);
+      gdk_region_subtract (invalidate_region, tmp_region);
+      gdk_region_destroy (tmp_region);
+    }
+  
+  gdk_window_invalidate_region (window, invalidate_region, TRUE);
+  gdk_region_destroy (invalidate_region);
+
+  /* We can guffaw scroll if we are a child window, and the parent
+   * does not extend beyond our edges. Otherwise, we use XCopyArea, then
+   * move any children later
+   */
   if (GDK_WINDOW_TYPE (window) == GDK_WINDOW_CHILD)
     {
       GdkWindowImplX11 *parent_impl = GDK_WINDOW_IMPL_X11 (obj->parent->impl);  
-      can_guffaw_scroll = (obj->x <= 0 &&
-			   obj->y <= 0 &&
-			   obj->x + impl->width >= parent_impl->width &&
-			   obj->y + impl->height >= parent_impl->height);
+      can_guffaw_scroll = ((dx == 0 || (obj->x <= 0 && obj->x + impl->width >= parent_impl->width)) &&
+			   (dy == 0 || (obj->y <= 0 && obj->y + impl->height >= parent_impl->height)));
     }
 
   if (!obj->children || !can_guffaw_scroll)
-    {
-      /* Use XCopyArea, then move any children later
-       */
-      GList *tmp_list;
-      GdkRegion *invalidate_region;
-      GdkRectangle dest_rect;
-
-      invalidate_region = gdk_region_rectangle (&impl->position_info.clip_rect);
-      
-      dest_rect = impl->position_info.clip_rect;
-      dest_rect.x += dx;
-      dest_rect.y += dy;
-      gdk_rectangle_intersect (&dest_rect, &impl->position_info.clip_rect, &dest_rect);
-
-      if (dest_rect.width > 0 && dest_rect.height > 0)
-	{
-	  GC gc;
-	  XGCValues values;
-	  GdkRegion *tmp_region;
-
-	  tmp_region = gdk_region_rectangle (&dest_rect);
-	  gdk_region_subtract (invalidate_region, tmp_region);
-	  gdk_region_destroy (tmp_region);
-	  
-	  gdk_window_queue_translation (window, dx, dy);
-
-	  values.graphics_exposures = True;
-	  gc = XCreateGC (GDK_WINDOW_XDISPLAY (window),
-                          GDK_WINDOW_XID (window),
-			  GCGraphicsExposures, &values);
-
-	  XCopyArea (GDK_WINDOW_XDISPLAY (window),
-		     GDK_WINDOW_XID (window),
-		     GDK_WINDOW_XID (window),
-		     gc,
-		     dest_rect.x - dx, dest_rect.y - dy,
-		     dest_rect.width, dest_rect.height,
-		     dest_rect.x, dest_rect.y);
-
-	  XFreeGC (GDK_WINDOW_XDISPLAY (window), gc);
-	}
-
-      gdk_window_invalidate_region (window, invalidate_region, TRUE);
-      gdk_region_destroy (invalidate_region);
-
-      tmp_list = obj->children;
-      while (tmp_list)
-	{
-	  GdkWindow * child = GDK_WINDOW (tmp_list->data);
-	  
-	  gdk_window_move (child, obj->x + dx, obj->y + dy);
-	  
-	  tmp_list = tmp_list->next;
-	}
-    }
+    gdk_window_copy_area_scroll (window, &dest_rect, dx, dy);
   else
-    {
-      /* Guffaw scroll
-       */
-      g_warning ("gdk_window_scroll(): guffaw scrolling not yet implemented");
-    }
+    gdk_window_guffaw_scroll (window, dx, dy);
 }
 
 void
@@ -278,38 +509,19 @@ _gdk_window_move_resize_child (GdkWindow *window,
   
   if (d_xoffset != 0 || d_yoffset != 0)
     {
-      gint new_x0, new_y0, new_x1, new_y1;
+      GdkRectangle new_position;
 
       gdk_window_set_static_gravities (window, TRUE);
 
       if (d_xoffset < 0 || d_yoffset < 0)
 	gdk_window_queue_translation (window, MIN (d_xoffset, 0), MIN (d_yoffset, 0));
-	
-      if (d_xoffset < 0)
-	{
-	  new_x0 = impl->position_info.x + d_xoffset;
-	  new_x1 = impl->position_info.x + impl->position_info.width;
-	}
-      else
-	{
-	  new_x0 = impl->position_info.x;
-	  new_x1 = impl->position_info.x + new_info.width + d_xoffset;
-	}
 
-      if (d_yoffset < 0)
-	{
-	  new_y0 = impl->position_info.y + d_yoffset;
-	  new_y1 = impl->position_info.y + impl->position_info.height;
-	}
-      else
-	{
-	  new_y0 = impl->position_info.y;
-	  new_y1 = impl->position_info.y + new_info.height + d_yoffset;
-	}
+      compute_intermediate_position (&impl->position_info, &new_info, d_xoffset, d_yoffset,
+				     &new_position);
       
       XMoveResizeWindow (GDK_WINDOW_XDISPLAY (window),
 			 GDK_WINDOW_XID (window),
-			 new_x0, new_y0, new_x1 - new_x0, new_y1 - new_y0);
+			 new_position.x, new_position.y, new_position.width, new_position.height);
       
       tmp_list = obj->children;
       while (tmp_list)
@@ -320,7 +532,7 @@ _gdk_window_move_resize_child (GdkWindow *window,
 
       XMoveWindow (GDK_WINDOW_XDISPLAY (window),
 		   GDK_WINDOW_XID (window),
-		   new_x0 + dx, new_y0 + dy);
+		   new_position.x + dx, new_position.y + dy);
       
       if (d_xoffset > 0 || d_yoffset > 0)
 	gdk_window_queue_translation (window, MAX (d_xoffset, 0), MAX (d_yoffset, 0));
@@ -400,7 +612,7 @@ gdk_window_compute_position (GdkWindowImplX11   *window,
   
   info->big = FALSE;
   
-  if (window->width <= 32768)
+  if (window->width <= 32767)
     {
       info->width = window->width;
       info->x = parent_pos->x + wrapper->x - parent_pos->x11_x;
@@ -408,19 +620,19 @@ gdk_window_compute_position (GdkWindowImplX11   *window,
   else
     {
       info->big = TRUE;
-      info->width = 32768;
+      info->width = 32767;
       if (parent_pos->x + wrapper->x < -16384)
 	{
 	  if (parent_pos->x + wrapper->x + window->width < 16384)
-	    info->x = parent_pos->x + wrapper->x + window->width - 32768 - parent_pos->x11_x;
+	    info->x = parent_pos->x + wrapper->x + window->width - info->width - parent_pos->x11_x;
 	  else
-	    info->x = -16384 - parent_pos->x11_y;
+	    info->x = -16384 - parent_pos->x11_x;
 	}
       else
 	info->x = parent_pos->x + wrapper->x - parent_pos->x11_x;
     }
 
-  if (window->height <= 32768)
+  if (window->height <= 32767)
     {
       info->height = window->height;
       info->y = parent_pos->y + wrapper->y - parent_pos->x11_y;
@@ -428,11 +640,11 @@ gdk_window_compute_position (GdkWindowImplX11   *window,
   else
     {
       info->big = TRUE;
-      info->height = 32768;
+      info->height = 32767;
       if (parent_pos->y + wrapper->y < -16384)
 	{
 	  if (parent_pos->y + wrapper->y + window->height < 16384)
-	    info->y = parent_pos->y + wrapper->y + window->height - 32768 - parent_pos->x11_y;
+	    info->y = parent_pos->y + wrapper->y + window->height - info->height - parent_pos->x11_y;
 	  else
 	    info->y = -16384 - parent_pos->x11_y;
 	}
@@ -576,36 +788,17 @@ gdk_window_premove (GdkWindow          *window,
   
   if (d_xoffset != 0 || d_yoffset != 0)
     {
-      gint new_x0, new_y0, new_x1, new_y1;
+      GdkRectangle new_position;
 
       if (d_xoffset < 0 || d_yoffset < 0)
 	gdk_window_queue_translation (window, MIN (d_xoffset, 0), MIN (d_yoffset, 0));
-	
-      if (d_xoffset < 0)
-	{
-	  new_x0 = impl->position_info.x + d_xoffset;
-	  new_x1 = impl->position_info.x + impl->position_info.width;
-	}
-      else
-	{
-	  new_x0 = impl->position_info.x;
-	  new_x1 = impl->position_info.x + new_info.width + d_xoffset;
-	}
 
-      if (d_yoffset < 0)
-	{
-	  new_y0 = impl->position_info.y + d_yoffset;
-	  new_y1 = impl->position_info.y + impl->position_info.height;
-	}
-      else
-	{
-	  new_y0 = impl->position_info.y;
-	  new_y1 = impl->position_info.y + new_info.height + d_yoffset;
-	}
+      compute_intermediate_position (&impl->position_info, &new_info, d_xoffset, d_yoffset,
+				     &new_position);
 
       XMoveResizeWindow (GDK_DRAWABLE_XDISPLAY (window),
 			 GDK_DRAWABLE_XID (window),
-			 new_x0, new_y0, new_x1 - new_x0, new_y1 - new_y0);
+			 new_position.x, new_position.y, new_position.width, new_position.height);
     }
 
   tmp_list = obj->children;
@@ -667,20 +860,134 @@ gdk_window_postmove (GdkWindow          *window,
     }
 }
 
+Bool
+expose_serial_predicate (Display *xdisplay,
+			 XEvent  *xev,
+			 XPointer arg)
+{
+  gulong *serial = (gulong *)arg;
+
+  if (xev->xany.type == Expose)
+    *serial = MIN (*serial, xev->xany.serial);
+
+  return False;
+}
+
+/* Find oldest possible serial for an outstanding expose event
+ */
+static gulong
+find_current_serial (Display *xdisplay)
+{
+  XEvent xev;
+  gulong serial = NextRequest (xdisplay);
+  
+  XSync (xdisplay, False);
+
+  XCheckIfEvent (xdisplay, &xev, expose_serial_predicate, (XPointer)&serial);
+
+  return serial;
+}
+
+static void
+queue_delete_link (GQueue *queue,
+		   GList  *link)
+{
+  if (queue->tail == link)
+    queue->tail = link->prev;
+  
+  queue->head = g_list_remove_link (queue->head, link);
+  g_list_free_1 (link);
+  queue->length--;
+}
+
+static void
+queue_item_free (GdkWindowQueueItem *item)
+{
+  g_object_unref (item->window);
+  
+  if (item->type == GDK_WINDOW_QUEUE_ANTIEXPOSE)
+    gdk_region_destroy (item->u.antiexpose.area);
+  
+  g_free (item);
+}
+
+static void
+gdk_window_queue (GdkWindow          *window,
+		  GdkWindowQueueItem *item)
+{
+  GdkDisplayX11 *display_x11 = GDK_DISPLAY_X11 (GDK_WINDOW_DISPLAY (window));
+  
+  if (!display_x11->translate_queue)
+    display_x11->translate_queue = g_queue_new ();
+
+  /* Keep length of queue finite by, if it grows too long,
+   * figuring out the latest relevant serial and discarding
+   * irrelevant queue items.
+   */
+  if (display_x11->translate_queue->length >= 64)
+    {
+      gulong serial = find_current_serial (GDK_WINDOW_XDISPLAY (window));
+      GList *tmp_list = display_x11->translate_queue->head;
+      
+      while (tmp_list)
+	{
+	  GdkWindowQueueItem *item = tmp_list->data;
+	  GList *next = tmp_list->next;
+	  
+	  if (serial > item->serial)
+	    {
+	      queue_delete_link (display_x11->translate_queue, tmp_list);
+	      queue_item_free (item);
+	    }
+
+	  tmp_list = next;
+	}
+    }
+
+  /* Catch the case where someone isn't processing events and there
+   * is an event stuck in the event queue with an old serial:
+   * If we can't reduce the queue length by the above method,
+   * discard anti-expose items. (We can't discard translate
+   * items 
+   */
+  if (display_x11->translate_queue->length >= 64)
+    {
+      GList *tmp_list = display_x11->translate_queue->head;
+      
+      while (tmp_list)
+	{
+	  GdkWindowQueueItem *item = tmp_list->data;
+	  GList *next = tmp_list->next;
+	  
+	  if (item->type == GDK_WINDOW_QUEUE_ANTIEXPOSE)
+	    {
+	      queue_delete_link (display_x11->translate_queue, tmp_list);
+	      queue_item_free (item);
+	    }
+
+	  tmp_list = next;
+	}
+    }
+      
+  g_object_ref (window);
+
+  item->window = window;
+  item->serial = NextRequest (GDK_WINDOW_XDISPLAY (window));
+  
+  g_queue_push_tail (display_x11->translate_queue, item);
+}
+
 static void
 gdk_window_queue_translation (GdkWindow *window,
 			      gint       dx,
 			      gint       dy)
 {
   GdkWindowQueueItem *item = g_new (GdkWindowQueueItem, 1);
-  item->window = window;
-  item->serial = NextRequest (GDK_WINDOW_XDISPLAY (window));
   item->type = GDK_WINDOW_QUEUE_TRANSLATE;
   item->u.translate.dx = dx;
   item->u.translate.dy = dy;
 
-  gdk_drawable_ref (window);
-  translate_queue = g_slist_append (translate_queue, item);
+  gdk_window_queue (window, item);
 }
 
 gboolean
@@ -688,13 +995,10 @@ _gdk_windowing_window_queue_antiexpose (GdkWindow *window,
 					GdkRegion *area)
 {
   GdkWindowQueueItem *item = g_new (GdkWindowQueueItem, 1);
-  item->window = window;
-  item->serial = NextRequest (GDK_WINDOW_XDISPLAY (window));
   item->type = GDK_WINDOW_QUEUE_ANTIEXPOSE;
   item->u.antiexpose.area = area;
 
-  gdk_drawable_ref (window);
-  translate_queue = g_slist_append (translate_queue, item);
+  gdk_window_queue (window, item);
 
   return TRUE;
 }
@@ -707,37 +1011,34 @@ _gdk_window_process_expose (GdkWindow    *window,
   GdkWindowImplX11 *impl;
   GdkRegion *invalidate_region = gdk_region_rectangle (area);
   GdkRegion *clip_region;
-  GSList *tmp_list = translate_queue;
-  
+  GdkDisplayX11 *display_x11 = GDK_DISPLAY_X11 (GDK_WINDOW_DISPLAY (window));
   impl = GDK_WINDOW_IMPL_X11 (GDK_WINDOW_OBJECT (window)->impl);
-  
-  while (tmp_list)
+      
+  if (display_x11->translate_queue)
     {
-      GdkWindowQueueItem *item = tmp_list->data;
-      tmp_list = tmp_list->next;
-
-      if (serial < item->serial)
+      GList *tmp_list = display_x11->translate_queue->head;
+      
+      while (tmp_list)
 	{
-	  if (item->window == window)
+	  GdkWindowQueueItem *item = tmp_list->data;
+	  tmp_list = tmp_list->next;
+	  
+	  if (serial < item->serial)
 	    {
-	      if (item->type == GDK_WINDOW_QUEUE_TRANSLATE)
-		gdk_region_offset (invalidate_region, item->u.translate.dx, item->u.translate.dy);
-	      else		/* anti-expose */
-		gdk_region_subtract (invalidate_region, item->u.antiexpose.area);
+	      if (item->window == window)
+		{
+		  if (item->type == GDK_WINDOW_QUEUE_TRANSLATE)
+		    gdk_region_offset (invalidate_region, item->u.translate.dx, item->u.translate.dy);
+		  else		/* anti-expose */
+		    gdk_region_subtract (invalidate_region, item->u.antiexpose.area);
+		}
+	    }
+	  else
+	    {
+	      queue_delete_link (display_x11->translate_queue, 
+				 display_x11->translate_queue->head);
+	      queue_item_free (item);
 	    }
-	}
-      else
-	{
-	  GSList *tmp_link = translate_queue;
-	  
-	  translate_queue = g_slist_remove_link (translate_queue, translate_queue);
-	  gdk_drawable_unref (item->window);
-
-	  if (item->type == GDK_WINDOW_QUEUE_ANTIEXPOSE)
-	    gdk_region_destroy (item->u.antiexpose.area);
-	  
-	  g_free (item);
-	  g_slist_free_1 (tmp_link);
 	}
     }
 
@@ -818,6 +1119,11 @@ gdk_window_clip_changed (GdkWindow *window, GdkRectangle *old_clip, GdkRectangle
   old_clip_region = gdk_region_rectangle (old_clip);
   new_clip_region = gdk_region_rectangle (new_clip);
 
+  /* We need to update this here because gdk_window_invalidate_region makes
+   * use if it (through gdk_drawable_get_visible_region
+   */
+  impl->position_info.clip_rect = *new_clip;
+  
   /* Trim invalid region of window to new clip rectangle
    */
   if (obj->update_area)
@@ -835,4 +1141,3 @@ gdk_window_clip_changed (GdkWindow *window, GdkRectangle *old_clip, GdkRectangle
   gdk_region_destroy (new_clip_region);
   gdk_region_destroy (old_clip_region);
 }
-
