@@ -54,7 +54,6 @@
 #include "gdkprivate-x11.h"
 #include "gdkdisplay-x11.h"
 #include "gdkscreen-x11.h"
-#include "gdkalias.h"
 
 typedef struct _GdkImagePrivateX11     GdkImagePrivateX11;
 
@@ -67,20 +66,47 @@ struct _GdkImagePrivateX11
 };
 
 static GList *image_list = NULL;
+static gpointer parent_class = NULL;
 
 static void gdk_x11_image_destroy (GdkImage      *image);
+static void gdk_image_init        (GdkImage      *image);
+static void gdk_image_class_init  (GdkImageClass *klass);
 static void gdk_image_finalize    (GObject       *object);
 
 #define PRIVATE_DATA(image) ((GdkImagePrivateX11 *) GDK_IMAGE (image)->windowing_data)
 
-G_DEFINE_TYPE (GdkImage, gdk_image, G_TYPE_OBJECT)
+GType
+gdk_image_get_type (void)
+{
+  static GType object_type = 0;
+
+  if (!object_type)
+    {
+      static const GTypeInfo object_info =
+      {
+        sizeof (GdkImageClass),
+        (GBaseInitFunc) NULL,
+        (GBaseFinalizeFunc) NULL,
+        (GClassInitFunc) gdk_image_class_init,
+        NULL,           /* class_finalize */
+        NULL,           /* class_data */
+        sizeof (GdkImage),
+        0,              /* n_preallocs */
+        (GInstanceInitFunc) gdk_image_init,
+      };
+      
+      object_type = g_type_register_static (G_TYPE_OBJECT,
+                                            "GdkImage",
+                                            &object_info, 0);
+    }
+  
+  return object_type;
+}
 
 static void
 gdk_image_init (GdkImage *image)
 {
-  image->windowing_data = G_TYPE_INSTANCE_GET_PRIVATE (image, 
-						       GDK_TYPE_IMAGE, 
-						       GdkImagePrivateX11);
+  image->windowing_data = g_new0 (GdkImagePrivateX11, 1);
 }
 
 static void
@@ -88,9 +114,9 @@ gdk_image_class_init (GdkImageClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  object_class->finalize = gdk_image_finalize;
+  parent_class = g_type_class_peek_parent (klass);
 
-  g_type_class_add_private (object_class, sizeof (GdkImagePrivateX11));
+  object_class->finalize = gdk_image_finalize;
 }
 
 static void
@@ -100,7 +126,7 @@ gdk_image_finalize (GObject *object)
 
   gdk_x11_image_destroy (image);
   
-  G_OBJECT_CLASS (gdk_image_parent_class)->finalize (object);
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 
@@ -131,10 +157,7 @@ _gdk_image_exit (void)
  * Return value: a new #GdkImage.
  **/
 GdkImage *
-gdk_image_new_bitmap (GdkVisual *visual, 
-		      gpointer   data, 
-		      gint       width, 
-		      gint       height)
+gdk_image_new_bitmap(GdkVisual *visual, gpointer data, gint width, gint height)
 {
   Visual *xvisual;
   GdkImage *image;
@@ -159,18 +182,42 @@ gdk_image_new_bitmap (GdkVisual *visual,
       xvisual = ((GdkVisualPrivate*) visual)->xvisual;
       private->ximage = XCreateImage (GDK_SCREEN_XDISPLAY (private->screen),
 				      xvisual, 1, XYBitmap,
-				      0, NULL, width, height, 8, 0);
-      private->ximage->data = data;
-      private->ximage->bitmap_bit_order = MSBFirst;
-      private->ximage->byte_order = MSBFirst;
+				      0, 0, width, height, 8, 0);
     }
   
+  private->ximage->data = data;
+  private->ximage->bitmap_bit_order = MSBFirst;
+  private->ximage->byte_order = MSBFirst;
   image->byte_order = MSBFirst;
   image->mem =  private->ximage->data;
   image->bpl = private->ximage->bytes_per_line;
   image->bpp = 1;
-  return image;
-} 
+  return(image);
+} /* gdk_image_new_bitmap() */
+
+/* 
+ * Desc: query the server for support for the MIT_SHM extension
+ * Return:  0 = not available
+ *          1 = shared XImage support available
+ *          2 = shared Pixmap support available also
+ */
+static int
+gdk_image_check_xshm(Display *display)
+{
+#ifdef USE_SHM
+  int major, minor, ignore;
+  Bool pixmaps;
+  
+  if (XQueryExtension(display, "MIT-SHM", &ignore, &ignore, &ignore)) 
+    {
+      if (XShmQueryVersion(display, &major, &minor, &pixmaps )==True) 
+	{
+	  return (pixmaps==True) ? 2 : 1;
+	}
+    }
+#endif /* USE_SHM */
+  return 0;
+}
 
 void
 _gdk_windowing_image_init (GdkDisplay *display)
@@ -179,23 +226,12 @@ _gdk_windowing_image_init (GdkDisplay *display)
   
   if (display_x11->use_xshm)
     {
-#ifdef USE_SHM
-      Display *xdisplay = display_x11->xdisplay;
-      int major, minor, event_base;
-      Bool pixmaps;
-  
-      if (XShmQueryExtension (xdisplay) &&
-	  XShmQueryVersion (xdisplay, &major, &minor, &pixmaps))
-	{
-	  display_x11->have_shm_pixmaps = pixmaps;
-	  event_base = XShmGetEventBase (xdisplay);
-
-	  gdk_x11_register_standard_event_type (display,
-						event_base, ShmNumberEvents);
-	}
+      gint res = gdk_image_check_xshm (GDK_DISPLAY_XDISPLAY (display));
+      
+      if (!res)
+	display_x11->use_xshm = False;
       else
-#endif /* USE_SHM */
-	display_x11->use_xshm = FALSE;
+	display_x11->have_shm_pixmaps = (res == 2);
     }
 }
 
@@ -294,7 +330,7 @@ _gdk_image_new_for_depth (GdkScreen    *screen,
 		}
 
 	      x_shm_info->readOnly = False;
-	      x_shm_info->shmaddr = shmat (x_shm_info->shmid, NULL, 0);
+	      x_shm_info->shmaddr = shmat (x_shm_info->shmid, 0, 0);
 	      private->ximage->data = x_shm_info->shmaddr;
 
 	      if (x_shm_info->shmaddr == (char*) -1)
@@ -326,7 +362,7 @@ _gdk_image_new_for_depth (GdkScreen    *screen,
 	       * we die in XShmAttach. In theory, a signal handler
 	       * could be set up.
 	       */
-	      shmctl (x_shm_info->shmid, IPC_RMID, NULL);		      
+	      shmctl (x_shm_info->shmid, IPC_RMID, 0);		      
 
 	      if (image)
 		image_list = g_list_prepend (image_list, image);
@@ -337,7 +373,7 @@ _gdk_image_new_for_depth (GdkScreen    *screen,
 	  break;
 	case GDK_IMAGE_NORMAL:
 	  private->ximage = XCreateImage (screen_x11->xdisplay, xvisual, depth,
-					  ZPixmap, 0, NULL, width, height, 32, 0);
+					  ZPixmap, 0, 0, width, height, 32, 0);
 
 	  /* Use malloc, not g_malloc here, because X will call free()
 	   * on this data
@@ -378,7 +414,7 @@ _gdk_image_new_for_depth (GdkScreen    *screen,
       if (x_shm_info->shmaddr != (char *)-1)
 	shmdt (x_shm_info->shmaddr);
       if (x_shm_info->shmid != -1) 
-	shmctl (x_shm_info->shmid, IPC_RMID, NULL);
+	shmctl (x_shm_info->shmid, IPC_RMID, 0);
       
       g_free (x_shm_info);
       private->x_shm_info = NULL;
@@ -497,6 +533,7 @@ _gdk_x11_copy_to_image (GdkDrawable    *drawable,
 #define UNGRAB() G_STMT_START {					\
     if (have_grab) {						\
       gdk_x11_display_ungrab (display);				\
+      XFlush (xdisplay);					\
       have_grab = FALSE; }					\
   } G_STMT_END
 
@@ -629,16 +666,17 @@ _gdk_x11_copy_to_image (GdkDrawable    *drawable,
 	}
     }
 
-  gdk_error_trap_pop ();
-
  out:
   
   if (have_grab)
     {				
       gdk_x11_display_ungrab (display);
+      XFlush (xdisplay);
       have_grab = FALSE;
     }
   
+  gdk_error_trap_pop ();
+
   if (success && !image)
     {
       /* We "succeeded", but could get no content for the image so return junk */
@@ -702,6 +740,12 @@ gdk_x11_image_destroy (GdkImage *image)
 
   private = PRIVATE_DATA (image);
 
+  if (private == NULL) /* This means that _gdk_image_exit() destroyed the
+                        * image already, and now we're called a second
+                        * time from _finalize()
+                        */
+    return;
+
   if (private->ximage)		/* Deal with failure of creation */
     {
       switch (image->type)
@@ -740,9 +784,10 @@ gdk_x11_image_destroy (GdkImage *image)
 	case GDK_IMAGE_FASTEST:
 	  g_assert_not_reached ();
 	}
-      
-      private->ximage = NULL;
     }
+
+  g_free (private);
+  image->windowing_data = NULL;
 }
 
 /**
@@ -809,6 +854,3 @@ _gdk_windowing_get_bits_for_depth (GdkDisplay *display,
   return -1;
 }
 
-
-#define __GDK_IMAGE_X11_C__
-#include "gdkaliasdef.c"
