@@ -30,7 +30,6 @@
 
 #include "gtkcontainer.h"
 #include "gtkprivate.h"
-#include "gtksignal.h"
 #include "gtkmain.h"
 #include "gtkmarshalers.h"
 #include "gtkwindow.h"
@@ -111,19 +110,20 @@ extern GObjectNotifyContext *_gtk_widget_child_property_notify_context;
 
 
 /* --- functions --- */
-GtkType
+GType
 gtk_container_get_type (void)
 {
-  static GtkType container_type = 0;
+  static GType container_type = 0;
 
   if (!container_type)
     {
-      static GTypeInfo container_info = {
+      static const GTypeInfo container_info =
+      {
 	sizeof (GtkContainerClass),
 	(GBaseInitFunc) gtk_container_base_class_init,
 	(GBaseFinalizeFunc) gtk_container_base_class_finalize,
 	(GClassInitFunc) gtk_container_class_init,
-	NULL        /* class_destroy */,
+	NULL        /* class_finalize */,
 	NULL        /* class_data */,
 	sizeof (GtkContainer),
 	0           /* n_preallocs */,
@@ -131,7 +131,9 @@ gtk_container_get_type (void)
 	NULL,       /* value_table */
       };
 
-      container_type = g_type_register_static (GTK_TYPE_WIDGET, "GtkContainer", &container_info, 0);
+      container_type =
+	g_type_register_static (GTK_TYPE_WIDGET, "GtkContainer", 
+				&container_info, G_TYPE_FLAG_ABSTRACT);
     }
 
   return container_type;
@@ -206,7 +208,7 @@ gtk_container_class_init (GtkContainerClass *class)
                                    PROP_BORDER_WIDTH,
                                    g_param_spec_uint ("border_width",
                                                       _("Border width"),
-                                                      _("The width of the empty border outside the containers children."),
+                                                      _("The width of the empty border outside the containers children"),
 						      0,
 						      G_MAXINT,
 						      0,
@@ -215,40 +217,44 @@ gtk_container_class_init (GtkContainerClass *class)
                                    PROP_CHILD,
                                    g_param_spec_object ("child",
                                                       _("Child"),
-                                                      _("Can be used to add a new child to the container."),
+                                                      _("Can be used to add a new child to the container"),
                                                       GTK_TYPE_WIDGET,
 						      G_PARAM_WRITABLE));
   container_signals[ADD] =
-    gtk_signal_new ("add",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkContainerClass, add),
-                    _gtk_marshal_VOID__OBJECT,
-		    GTK_TYPE_NONE, 1,
-                    GTK_TYPE_WIDGET);
+    g_signal_new ("add",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkContainerClass, add),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__OBJECT,
+		  G_TYPE_NONE, 1,
+		  GTK_TYPE_WIDGET);
   container_signals[REMOVE] =
-    gtk_signal_new ("remove",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkContainerClass, remove),
-                    _gtk_marshal_VOID__OBJECT,
-		    GTK_TYPE_NONE, 1,
-                    GTK_TYPE_WIDGET);
+    g_signal_new ("remove",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkContainerClass, remove),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__OBJECT,
+		  G_TYPE_NONE, 1,
+		  GTK_TYPE_WIDGET);
   container_signals[CHECK_RESIZE] =
-    gtk_signal_new ("check_resize",
-                    GTK_RUN_LAST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkContainerClass, check_resize),
-		    _gtk_marshal_VOID__VOID,
-		    GTK_TYPE_NONE, 0);
+    g_signal_new ("check_resize",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_LAST,
+		  G_STRUCT_OFFSET (GtkContainerClass, check_resize),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__VOID,
+		  G_TYPE_NONE, 0);
   container_signals[SET_FOCUS_CHILD] =
-    gtk_signal_new ("set-focus-child",
-                    GTK_RUN_FIRST,
-                    GTK_CLASS_TYPE (object_class),
-                    GTK_SIGNAL_OFFSET (GtkContainerClass, set_focus_child),
-                    _gtk_marshal_VOID__OBJECT,
-		    GTK_TYPE_NONE, 1,
-                    GTK_TYPE_WIDGET);
+    g_signal_new ("set-focus-child",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (GtkContainerClass, set_focus_child),
+		  NULL, NULL,
+		  _gtk_marshal_VOID__OBJECT,
+		  G_TYPE_NONE, 1,
+		  GTK_TYPE_WIDGET);
 }
 
 /**
@@ -257,16 +263,16 @@ gtk_container_class_init (GtkContainerClass *class)
  *
  * Returns the type of the children supported by the container.
  *
- * Note that this may return %GTK_TYPE_NONE to indicate that no more
+ * Note that this may return %G_TYPE_NONE to indicate that no more
  * children can be added, e.g. for a #GtkPaned which already has two 
  * children.
  *
- * Return value: a #GtkType.
+ * Return value: a #GType.
  **/
-GtkType
+GType
 gtk_container_child_type (GtkContainer *container)
 {
-  GtkType slot;
+  GType slot;
   GtkContainerClass *class;
 
   g_return_val_if_fail (GTK_IS_CONTAINER (container), 0);
@@ -275,7 +281,7 @@ gtk_container_child_type (GtkContainer *container)
   if (class->child_type)
     slot = class->child_type (container);
   else
-    slot = GTK_TYPE_NONE;
+    slot = G_TYPE_NONE;
 
   return slot;
 }
@@ -571,11 +577,11 @@ gtk_container_add_with_properties (GtkContainer *container,
   g_return_if_fail (GTK_IS_WIDGET (widget));
   g_return_if_fail (widget->parent == NULL);
 
-  gtk_widget_ref (GTK_WIDGET (container));
-  gtk_widget_ref (widget);
+  g_object_ref (container);
+  g_object_ref (widget);
   gtk_widget_freeze_child_notify (widget);
 
-  gtk_signal_emit (GTK_OBJECT (container), container_signals[ADD], widget);
+  g_signal_emit (container, container_signals[ADD], 0, widget);
   if (widget->parent)
     {
       va_list var_args;
@@ -586,8 +592,8 @@ gtk_container_add_with_properties (GtkContainer *container,
     }
 
   gtk_widget_thaw_child_notify (widget);
-  gtk_widget_unref (widget);
-  gtk_widget_unref (GTK_WIDGET (container));
+  g_object_unref (widget);
+  g_object_unref (container);
 }
 
 void
@@ -625,58 +631,75 @@ gtk_container_child_get (GtkContainer      *container,
 }
 
 void
-gtk_container_class_install_child_property (GtkContainerClass *class,
+gtk_container_class_install_child_property (GtkContainerClass *cclass,
 					    guint              property_id,
 					    GParamSpec        *pspec)
 {
-  g_return_if_fail (GTK_IS_CONTAINER_CLASS (class));
+  g_return_if_fail (GTK_IS_CONTAINER_CLASS (cclass));
   g_return_if_fail (G_IS_PARAM_SPEC (pspec));
   if (pspec->flags & G_PARAM_WRITABLE)
-    g_return_if_fail (class->set_child_property != NULL);
+    g_return_if_fail (cclass->set_child_property != NULL);
   if (pspec->flags & G_PARAM_READABLE)
-    g_return_if_fail (class->get_child_property != NULL);
+    g_return_if_fail (cclass->get_child_property != NULL);
   g_return_if_fail (property_id > 0);
   g_return_if_fail (PARAM_SPEC_PARAM_ID (pspec) == 0);  /* paranoid */
   if (pspec->flags & (G_PARAM_CONSTRUCT | G_PARAM_CONSTRUCT_ONLY))
     g_return_if_fail ((pspec->flags & (G_PARAM_CONSTRUCT | G_PARAM_CONSTRUCT_ONLY)) == 0);
 
-  if (g_param_spec_pool_lookup (_gtk_widget_child_property_pool, pspec->name, G_OBJECT_CLASS_TYPE (class), FALSE))
+  if (g_param_spec_pool_lookup (_gtk_widget_child_property_pool, pspec->name, G_OBJECT_CLASS_TYPE (cclass), FALSE))
     {
-      g_warning (G_STRLOC ": class `%s' already contains a property named `%s'",
-		 G_OBJECT_CLASS_NAME (class),
+      g_warning (G_STRLOC ": class `%s' already contains a child property named `%s'",
+		 G_OBJECT_CLASS_NAME (cclass),
 		 pspec->name);
       return;
     }
   g_param_spec_ref (pspec);
   g_param_spec_sink (pspec);
   PARAM_SPEC_SET_PARAM_ID (pspec, property_id);
-  g_param_spec_pool_insert (_gtk_widget_child_property_pool, pspec, G_OBJECT_CLASS_TYPE (class));
+  g_param_spec_pool_insert (_gtk_widget_child_property_pool, pspec, G_OBJECT_CLASS_TYPE (cclass));
 }
 
+/**
+ * gtk_container_class_find_child_property:
+ * @cclass: a #GtkContainerClass
+ * @property_name: the name of the child property to find
+ * @returns: the #GParamSpec of the child property or %NULL if @class has ho
+ *   child property with that name.
+ *
+ * Finds a child property of a container class by name.
+ */
 GParamSpec*
-gtk_container_class_find_child_property (GObjectClass *class,
+gtk_container_class_find_child_property (GObjectClass *cclass,
 					 const gchar  *property_name)
 {
-  g_return_val_if_fail (GTK_IS_CONTAINER_CLASS (class), NULL);
+  g_return_val_if_fail (GTK_IS_CONTAINER_CLASS (cclass), NULL);
   g_return_val_if_fail (property_name != NULL, NULL);
 
   return g_param_spec_pool_lookup (_gtk_widget_child_property_pool,
 				   property_name,
-				   G_OBJECT_CLASS_TYPE (class),
+				   G_OBJECT_CLASS_TYPE (cclass),
 				   TRUE);
 }
 
-GParamSpec** /* free result */
-gtk_container_class_list_child_properties (GObjectClass *class,
+/**
+ * gtk_container_class_list_child_properties:
+ * @cclass: a #GtkContainerClass
+ * @n_properties: location to return the number of child properties found
+ * @returns: an newly allocated array of #GParamSpec*. The array must be freed with g_free().
+ *
+ * Returns all child properties of a container class.
+ */
+GParamSpec**
+gtk_container_class_list_child_properties (GObjectClass *cclass,
 					   guint        *n_properties)
 {
   GParamSpec **pspecs;
   guint n;
 
-  g_return_val_if_fail (GTK_IS_CONTAINER_CLASS (class), NULL);
+  g_return_val_if_fail (GTK_IS_CONTAINER_CLASS (cclass), NULL);
 
   pspecs = g_param_spec_pool_list (_gtk_widget_child_property_pool,
-				   G_OBJECT_CLASS_TYPE (class),
+				   G_OBJECT_CLASS_TYPE (cclass),
 				   &n);
   if (n_properties)
     *n_properties = n;
@@ -688,14 +711,14 @@ static void
 gtk_container_add_unimplemented (GtkContainer     *container,
 				 GtkWidget        *widget)
 {
-  g_warning ("GtkContainerClass::add not implemented for `%s'", gtk_type_name (GTK_OBJECT_TYPE (container)));
+  g_warning ("GtkContainerClass::add not implemented for `%s'", g_type_name (G_TYPE_FROM_INSTANCE (container)));
 }
 
 static void
 gtk_container_remove_unimplemented (GtkContainer     *container,
 				    GtkWidget        *widget)
 {
-  g_warning ("GtkContainerClass::remove not implemented for `%s'", gtk_type_name (GTK_OBJECT_TYPE (container)));
+  g_warning ("GtkContainerClass::remove not implemented for `%s'", g_type_name (G_TYPE_FROM_INSTANCE (container)));
 }
 
 static void
@@ -856,7 +879,7 @@ gtk_container_add (GtkContainer *container,
       return;
     }
 
-  gtk_signal_emit (GTK_OBJECT (container), container_signals[ADD], widget);
+  g_signal_emit (container, container_signals[ADD], 0, widget);
 }
 
 /**
@@ -879,7 +902,7 @@ gtk_container_remove (GtkContainer *container,
   g_return_if_fail (GTK_IS_WIDGET (widget));
   g_return_if_fail (widget->parent == GTK_WIDGET (container));
   
-  gtk_signal_emit (GTK_OBJECT (container), container_signals[REMOVE], widget);
+  g_signal_emit (container, container_signals[REMOVE], 0, widget);
 }
 
 void
@@ -1074,7 +1097,7 @@ gtk_container_check_resize (GtkContainer *container)
 {
   g_return_if_fail (GTK_IS_CONTAINER (container));
   
-  gtk_signal_emit (GTK_OBJECT (container), container_signals[CHECK_RESIZE]);
+  g_signal_emit (container, container_signals[CHECK_RESIZE], 0);
 }
 
 static void
@@ -1196,12 +1219,12 @@ gtk_container_foreach_unmarshal (GtkWidget *child,
   
   /* first argument */
   args[0].name = NULL;
-  args[0].type = GTK_OBJECT_TYPE (child);
+  args[0].type = G_TYPE_FROM_INSTANCE (child);
   GTK_VALUE_OBJECT (args[0]) = GTK_OBJECT (child);
   
   /* location for return value */
   args[1].name = NULL;
-  args[1].type = GTK_TYPE_NONE;
+  args[1].type = G_TYPE_NONE;
   
   fdata->callback (fdata->container, fdata->callback_data, 1, args);
 }
@@ -1244,7 +1267,7 @@ gtk_container_set_focus_child (GtkContainer *container,
   if (widget)
     g_return_if_fail (GTK_IS_WIDGET (widget));
 
-  gtk_signal_emit (GTK_OBJECT (container), container_signals[SET_FOCUS_CHILD], widget);
+  g_signal_emit (container, container_signals[SET_FOCUS_CHILD], 0, widget);
 }
 
 /**
@@ -1303,7 +1326,7 @@ gtk_container_child_default_composite_name (GtkContainer *container,
 			&data);
   
   name = g_strdup_printf ("%s-%u",
-			  gtk_type_name (GTK_OBJECT_TYPE (child)),
+			  g_type_name (G_TYPE_FROM_INSTANCE (child)),
 			  data.index);
 
   return name;
@@ -1325,7 +1348,7 @@ _gtk_container_child_composite_name (GtkContainer *container,
       if (!quark_composite_name)
 	quark_composite_name = g_quark_from_static_string ("gtk-composite-name");
 
-      name = gtk_object_get_data_by_id (GTK_OBJECT (child), quark_composite_name);
+      name = g_object_get_qdata (G_OBJECT (child), quark_composite_name);
       if (!name)
 	{
 	  GtkContainerClass *class;
@@ -1354,10 +1377,10 @@ gtk_container_real_set_focus_child (GtkContainer     *container,
   if (child != container->focus_child)
     {
       if (container->focus_child)
-	gtk_widget_unref (container->focus_child);
+	g_object_unref (container->focus_child);
       container->focus_child = child;
       if (container->focus_child)
-	gtk_widget_ref (container->focus_child);
+	g_object_ref (container->focus_child);
     }
 
 
@@ -1367,14 +1390,14 @@ gtk_container_real_set_focus_child (GtkContainer     *container,
     {
       GtkAdjustment *adjustment;
       
-      adjustment = gtk_object_get_data_by_id (GTK_OBJECT (container), vadjustment_key_id);
+      adjustment = g_object_get_qdata (G_OBJECT (container), vadjustment_key_id);
       if (adjustment)
 	gtk_adjustment_clamp_page (adjustment,
 				   container->focus_child->allocation.y,
 				   (container->focus_child->allocation.y +
 				    container->focus_child->allocation.height));
 
-      adjustment = gtk_object_get_data_by_id (GTK_OBJECT (container), hadjustment_key_id);
+      adjustment = g_object_get_qdata (G_OBJECT (container), hadjustment_key_id);
       if (adjustment)
 	gtk_adjustment_clamp_page (adjustment,
 				   container->focus_child->allocation.x,
@@ -1935,8 +1958,8 @@ chain_widget_destroyed (GtkWidget *widget,
 
   chain = g_list_remove (chain, widget);
 
-  g_signal_handlers_disconnect_by_func (G_OBJECT (widget),
-                                        (gpointer) chain_widget_destroyed,
+  g_signal_handlers_disconnect_by_func (widget,
+                                        chain_widget_destroyed,
                                         user_data);
   
   g_object_set_data (G_OBJECT (container),
@@ -1986,10 +2009,10 @@ gtk_container_set_focus_chain (GtkContainer *container,
 
       chain = g_list_prepend (chain, tmp_list->data);
 
-      gtk_signal_connect (GTK_OBJECT (tmp_list->data),
-                          "destroy",
-                          GTK_SIGNAL_FUNC (chain_widget_destroyed),
-                          container);
+      g_signal_connect (tmp_list->data,
+                        "destroy",
+                        G_CALLBACK (chain_widget_destroyed),
+                        container);
       
       tmp_list = g_list_next (tmp_list);
     }
@@ -2062,8 +2085,8 @@ gtk_container_unset_focus_chain (GtkContainer  *container)
       tmp_list = chain;
       while (tmp_list != NULL)
         {
-          g_signal_handlers_disconnect_by_func (G_OBJECT (tmp_list->data),
-                                                (gpointer) chain_widget_destroyed,
+          g_signal_handlers_disconnect_by_func (tmp_list->data,
+                                                chain_widget_destroyed,
                                                 container);
           
           tmp_list = g_list_next (tmp_list);
@@ -2082,12 +2105,12 @@ gtk_container_set_focus_vadjustment (GtkContainer  *container,
     g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
 
   if (adjustment)
-    gtk_object_ref (GTK_OBJECT(adjustment));
+    g_object_ref (adjustment);
 
-  gtk_object_set_data_by_id_full (GTK_OBJECT (container),
-				  vadjustment_key_id,
-				  adjustment,
-				  (GtkDestroyNotify) gtk_object_unref);
+  g_object_set_qdata_full (G_OBJECT (container),
+			   vadjustment_key_id,
+			   adjustment,
+			   g_object_unref);
 }
 
 /**
@@ -2107,8 +2130,7 @@ gtk_container_get_focus_vadjustment (GtkContainer *container)
     
   g_return_val_if_fail (GTK_IS_CONTAINER (container), NULL);
 
-  vadjustment = gtk_object_get_data_by_id (GTK_OBJECT (container),
-					   vadjustment_key_id);
+  vadjustment = g_object_get_qdata (G_OBJECT (container), vadjustment_key_id);
 
   return vadjustment;
 }
@@ -2122,12 +2144,12 @@ gtk_container_set_focus_hadjustment (GtkContainer  *container,
     g_return_if_fail (GTK_IS_ADJUSTMENT (adjustment));
 
   if (adjustment)
-    gtk_object_ref (GTK_OBJECT (adjustment));
+    g_object_ref (adjustment);
 
-  gtk_object_set_data_by_id_full (GTK_OBJECT (container),
-				  hadjustment_key_id,
-				  adjustment,
-				  (GtkDestroyNotify) gtk_object_unref);
+  g_object_set_qdata_full (G_OBJECT (container),
+			   hadjustment_key_id,
+			   adjustment,
+			   g_object_unref);
 }
 
 /**
@@ -2147,8 +2169,7 @@ gtk_container_get_focus_hadjustment (GtkContainer *container)
 
   g_return_val_if_fail (GTK_IS_CONTAINER (container), NULL);
 
-  hadjustment = gtk_object_get_data_by_id (GTK_OBJECT (container),
-		  			   hadjustment_key_id);
+  hadjustment = g_object_get_qdata (G_OBJECT (container), hadjustment_key_id);
 
   return hadjustment;
 }
@@ -2278,7 +2299,7 @@ gtk_container_propagate_expose (GtkContainer   *container,
 				GtkWidget      *child,
 				GdkEventExpose *event)
 {
-  GdkEventExpose child_event;
+  GdkEvent *child_event;
 
   g_return_if_fail (GTK_IS_CONTAINER (container));
   g_return_if_fail (GTK_IS_WIDGET (child));
@@ -2290,14 +2311,16 @@ gtk_container_propagate_expose (GtkContainer   *container,
       GTK_WIDGET_NO_WINDOW (child) &&
       (child->window == event->window))
     {
-      child_event = *event;
+      child_event = gdk_event_new (GDK_EXPOSE);
+      child_event->expose = *event;
+      g_object_ref (child_event->expose.window);
 
-      child_event.region = gtk_widget_region_intersect (child, event->region);
-      if (!gdk_region_empty (child_event.region))
+      child_event->expose.region = gtk_widget_region_intersect (child, event->region);
+      if (!gdk_region_empty (child_event->expose.region))
 	{
-	  gdk_region_get_clipbox (child_event.region, &child_event.area);
-	  gtk_widget_send_expose (child, (GdkEvent *)&child_event);
+	  gdk_region_get_clipbox (child_event->expose.region, &child_event->expose.area);
+	  gtk_widget_send_expose (child, child_event);
 	}
-      gdk_region_destroy (child_event.region);
+      gdk_event_free (child_event);
     }
 }
