@@ -24,7 +24,6 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include <ctype.h>
 #include <string.h>
 #include "gdk/gdkkeysyms.h"
 #include "gdk/gdki18n.h"
@@ -134,16 +133,18 @@ gtk_old_editable_get_type (void)
 
   if (!old_editable_type)
     {
-      static const GtkTypeInfo old_editable_info =
+      static const GTypeInfo old_editable_info =
       {
-	"GtkOldEditable",
-	sizeof (GtkOldEditable),
 	sizeof (GtkOldEditableClass),
-	(GtkClassInitFunc) gtk_old_editable_class_init,
-	(GtkObjectInitFunc) gtk_old_editable_init,
-	/* reserved_1 */ NULL,
-	/* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
+        (GClassInitFunc) gtk_old_editable_class_init,
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
+	sizeof (GtkOldEditable),
+	0,		/* n_preallocs */
+        (GInstanceInitFunc) gtk_old_editable_init,
+	NULL,		/* value_table */
       };
 
       static const GInterfaceInfo editable_info =
@@ -153,7 +154,8 @@ gtk_old_editable_get_type (void)
 	NULL			                                 /* interface_data */
       };
 
-      old_editable_type = gtk_type_unique (GTK_TYPE_WIDGET, &old_editable_info);
+      old_editable_type = g_type_register_static (GTK_TYPE_WIDGET, "GtkOldEditable",
+						  &old_editable_info, G_TYPE_FLAG_ABSTRACT);
       g_type_add_interface_static (old_editable_type,
 				   GTK_TYPE_EDITABLE,
 				   &editable_info);
@@ -551,7 +553,7 @@ gtk_old_editable_selection_clear (GtkWidget         *widget,
   
   /* Let the selection handling code know that the selection
    * has been changed, since we've overriden the default handler */
-  if (!gtk_selection_clear (widget, event))
+  if (!parent_class->selection_clear_event (widget, event))
     return FALSE;
   
   if (old_editable->has_selection)
@@ -708,11 +710,23 @@ old_editable_text_received_cb (GtkClipboard *clipboard,
   g_object_unref (G_OBJECT (old_editable));
 }
 
+/**
+ * gtk_old_editable_claim_selection:
+ * @old_editable: a #GtkOldEditable
+ * @claim: if %TRUE, claim ownership of the selection, if %FALSE, give
+ *   up ownership
+ * @time: timestamp for this operation
+ * 
+ * Claims or gives up ownership of the selection.
+ */
 void
 gtk_old_editable_claim_selection (GtkOldEditable *old_editable, 
 				  gboolean        claim, 
 				  guint32         time)
 {
+  GtkWidget *widget = GTK_WIDGET (old_editable);
+  GdkDisplay *display = gtk_widget_get_display (widget);
+  
   g_return_if_fail (GTK_IS_OLD_EDITABLE (old_editable));
   g_return_if_fail (GTK_WIDGET_REALIZED (old_editable));
 
@@ -720,13 +734,16 @@ gtk_old_editable_claim_selection (GtkOldEditable *old_editable,
   
   if (claim)
     {
-      if (gtk_selection_owner_set (GTK_WIDGET (old_editable), GDK_SELECTION_PRIMARY, time))
+      if (gtk_selection_owner_set_for_display (display, widget,
+					       GDK_SELECTION_PRIMARY, time))
 	old_editable->has_selection = TRUE;
     }
   else
     {
-      if (gdk_selection_owner_get (GDK_SELECTION_PRIMARY) == GTK_WIDGET (old_editable)->window)
-	gtk_selection_owner_set (NULL, GDK_SELECTION_PRIMARY, time);
+      if (gdk_selection_owner_get_for_display (display, GDK_SELECTION_PRIMARY) == widget->window)
+	gtk_selection_owner_set_for_display (display,
+					     NULL,
+					     GDK_SELECTION_PRIMARY, time);
     }
 }
 
@@ -793,7 +810,10 @@ gtk_old_editable_real_copy_clipboard (GtkOldEditable *old_editable)
 
       if (text)
 	{
-	  gtk_clipboard_set_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD), text, -1);
+	  GtkClipboard *clipboard = gtk_widget_get_clipboard (GTK_WIDGET (old_editable),
+							      GDK_SELECTION_CLIPBOARD);
+	  
+	  gtk_clipboard_set_text (clipboard, text, -1);
 	  g_free (text);
 	}
     }
@@ -802,11 +822,19 @@ gtk_old_editable_real_copy_clipboard (GtkOldEditable *old_editable)
 static void
 gtk_old_editable_real_paste_clipboard (GtkOldEditable *old_editable)
 {
+  GtkClipboard *clipboard = gtk_widget_get_clipboard (GTK_WIDGET (old_editable), 
+						      GDK_SELECTION_CLIPBOARD);
+
   g_object_ref (G_OBJECT (old_editable));
-  gtk_clipboard_request_text (gtk_clipboard_get (GDK_SELECTION_CLIPBOARD),
-			      old_editable_text_received_cb, old_editable);
+  gtk_clipboard_request_text (clipboard, old_editable_text_received_cb, old_editable);
 }
 
+/**
+ * gtk_old_editable_changed:
+ * @old_editable: a #GtkOldEditable
+ *
+ * Emits the ::changed signal on @old_editable.
+ */
 void
 gtk_old_editable_changed (GtkOldEditable *old_editable)
 {
