@@ -38,11 +38,11 @@ extern "C" {
 
 
 #define GTK_TYPE_OPTION_MENU              (gtk_option_menu_get_type ())
-#define GTK_OPTION_MENU(obj)              (GTK_CHECK_CAST ((obj), GTK_TYPE_OPTION_MENU, GtkOptionMenu))
-#define GTK_OPTION_MENU_CLASS(klass)      (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_OPTION_MENU, GtkOptionMenuClass))
-#define GTK_IS_OPTION_MENU(obj)           (GTK_CHECK_TYPE ((obj), GTK_TYPE_OPTION_MENU))
-#define GTK_IS_OPTION_MENU_CLASS(klass)   (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_OPTION_MENU))
-#define GTK_OPTION_MENU_GET_CLASS(obj)    (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_OPTION_MENU, GtkOptionMenuClass))
+#define GTK_OPTION_MENU(obj)              (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_OPTION_MENU, GtkOptionMenu))
+#define GTK_OPTION_MENU_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_OPTION_MENU, GtkOptionMenuClass))
+#define GTK_IS_OPTION_MENU(obj)           (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_OPTION_MENU))
+#define GTK_IS_OPTION_MENU_CLASS(klass)   (G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_OPTION_MENU))
+#define GTK_OPTION_MENU_GET_CLASS(obj)    (G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_OPTION_MENU, GtkOptionMenuClass))
 
 
 typedef struct _GtkOptionMenu       GtkOptionMenu;
@@ -73,7 +73,7 @@ struct _GtkOptionMenuClass
 };
 
 
-GtkType    gtk_option_menu_get_type    (void) G_GNUC_CONST;
+GType      gtk_option_menu_get_type    (void) G_GNUC_CONST;
 GtkWidget* gtk_option_menu_new         (void);
 GtkWidget* gtk_option_menu_get_menu    (GtkOptionMenu *option_menu);
 void       gtk_option_menu_set_menu    (GtkOptionMenu *option_menu,
