@@ -39,6 +39,8 @@ extern "C" {
 typedef struct _GtkTargetList    GtkTargetList;
 typedef struct _GtkTargetEntry   GtkTargetEntry;
 
+#define GTK_TYPE_SELECTION_DATA (gtk_selection_data_get_type ())
+
 /* The contents of a selection are returned in a GtkSelectionData
  * structure. selection/target identify the request.  type specifies
  * the type of the return; if length < 0, and the data should be
@@ -52,12 +54,13 @@ typedef struct _GtkTargetEntry   GtkTargetEntry;
 
 struct _GtkSelectionData
 {
-  GdkAtom selection;
-  GdkAtom target;
-  GdkAtom type;
-  gint	  format;
-  guchar *data;  
-  gint	  length;
+  GdkAtom	selection;
+  GdkAtom	target;
+  GdkAtom	type;
+  gint		format;
+  guchar       *data;  
+  gint		length;
+  GdkDisplay   *display;
 };
 
 struct _GtkTargetEntry {
@@ -103,9 +106,14 @@ gboolean       gtk_target_list_find      (GtkTargetList  *list,
 
 /* Public interface */
 
-gboolean gtk_selection_owner_set     (GtkWidget            *widget,
-				      GdkAtom               selection,
-				      guint32               time);
+gboolean gtk_selection_owner_set             (GtkWidget  *widget,
+					      GdkAtom     selection,
+					      guint32     time);
+gboolean gtk_selection_owner_set_for_display (GdkDisplay *display,
+					      GtkWidget  *widget,
+					      GdkAtom     selection,
+					      guint32     time);
+
 void     gtk_selection_add_target    (GtkWidget            *widget,
 				      GdkAtom               selection,
 				      GdkAtom               target,
@@ -126,7 +134,8 @@ void     gtk_selection_data_set      (GtkSelectionData     *selection_data,
 				      const guchar         *data,
 				      gint                  length);
 gboolean gtk_selection_data_set_text (GtkSelectionData     *selection_data,
-				      const guchar         *str);
+				      const gchar          *str,
+				      gint                  len);
 guchar * gtk_selection_data_get_text (GtkSelectionData     *selection_data);
 
 gboolean gtk_selection_data_get_targets          (GtkSelectionData  *selection_data,
@@ -142,16 +151,18 @@ void gtk_selection_remove_all      (GtkWidget *widget);
 
 gboolean gtk_selection_clear		  (GtkWidget 	     *widget,
 					   GdkEventSelection *event);
-gboolean gtk_selection_request		  (GtkWidget  	     *widget,
+gboolean _gtk_selection_request		  (GtkWidget  	     *widget,
 					   GdkEventSelection *event);
-gboolean gtk_selection_incr_event	  (GdkWindow         *window,
+gboolean _gtk_selection_incr_event	  (GdkWindow         *window,
 					   GdkEventProperty  *event);
-gboolean gtk_selection_notify		  (GtkWidget         *widget,
+gboolean _gtk_selection_notify		  (GtkWidget         *widget,
 					   GdkEventSelection *event);
-gboolean gtk_selection_property_notify	  (GtkWidget         *widget,
+gboolean _gtk_selection_property_notify	  (GtkWidget         *widget,
 					   GdkEventProperty  *event);
-GtkSelectionData *gtk_selection_data_copy (GtkSelectionData *data);
-void		  gtk_selection_data_free (GtkSelectionData *data);
+
+GType             gtk_selection_data_get_type (void);
+GtkSelectionData *gtk_selection_data_copy     (GtkSelectionData *data);
+void		  gtk_selection_data_free     (GtkSelectionData *data);
 
 
 
