@@ -29,11 +29,11 @@ extern "C" {
 #endif /* __cplusplus */
 
 #define GTK_TYPE_TREE_STORE			(gtk_tree_store_get_type ())
-#define GTK_TREE_STORE(obj)			(GTK_CHECK_CAST ((obj), GTK_TYPE_TREE_STORE, GtkTreeStore))
-#define GTK_TREE_STORE_CLASS(klass)		(GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_TREE_STORE, GtkTreeStoreClass))
-#define GTK_IS_TREE_STORE(obj)			(GTK_CHECK_TYPE ((obj), GTK_TYPE_TREE_STORE))
-#define GTK_IS_TREE_STORE_CLASS(klass)		(GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_TREE_STORE))
-#define GTK_TREE_STORE_GET_CLASS(obj)           (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_TREE_STORE, GtkTreeStoreClass))
+#define GTK_TREE_STORE(obj)			(G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_TREE_STORE, GtkTreeStore))
+#define GTK_TREE_STORE_CLASS(klass)		(G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_TREE_STORE, GtkTreeStoreClass))
+#define GTK_IS_TREE_STORE(obj)			(G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_TREE_STORE))
+#define GTK_IS_TREE_STORE_CLASS(klass)		(G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_TYPE_TREE_STORE))
+#define GTK_TREE_STORE_GET_CLASS(obj)		(G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_TYPE_TREE_STORE, GtkTreeStoreClass))
 
 typedef struct _GtkTreeStore       GtkTreeStore;
 typedef struct _GtkTreeStoreClass  GtkTreeStoreClass;
@@ -59,10 +59,16 @@ struct _GtkTreeStore
 struct _GtkTreeStoreClass
 {
   GObjectClass parent_class;
+
+  /* Padding for future expansion */
+  void (*_gtk_reserved1) (void);
+  void (*_gtk_reserved2) (void);
+  void (*_gtk_reserved3) (void);
+  void (*_gtk_reserved4) (void);
 };
 
 
-GtkType       gtk_tree_store_get_type         (void);
+GType         gtk_tree_store_get_type         (void);
 GtkTreeStore *gtk_tree_store_new              (gint          n_columns,
 					       ...);
 GtkTreeStore *gtk_tree_store_newv             (gint          n_columns,
@@ -83,7 +89,7 @@ void          gtk_tree_store_set              (GtkTreeStore *tree_store,
 void          gtk_tree_store_set_valist       (GtkTreeStore *tree_store,
 					       GtkTreeIter  *iter,
 					       va_list       var_args);
-void          gtk_tree_store_remove           (GtkTreeStore *tree_store,
+gboolean      gtk_tree_store_remove           (GtkTreeStore *tree_store,
 					       GtkTreeIter  *iter);
 void          gtk_tree_store_insert           (GtkTreeStore *tree_store,
 					       GtkTreeIter  *iter,
@@ -109,6 +115,17 @@ gboolean      gtk_tree_store_is_ancestor      (GtkTreeStore *tree_store,
 gint          gtk_tree_store_iter_depth       (GtkTreeStore *tree_store,
 					       GtkTreeIter  *iter);
 void          gtk_tree_store_clear            (GtkTreeStore *tree_store);
+gboolean      gtk_tree_store_iter_is_valid    (GtkTreeStore *tree_store,
+                                               GtkTreeIter  *iter);
+void          gtk_tree_store_reorder          (GtkTreeStore *tree_store,
+                                               GtkTreeIter  *parent,
+                                               gint         *new_order);
+void          gtk_tree_store_swap             (GtkTreeStore *tree_store,
+                                               GtkTreeIter  *a,
+                                               GtkTreeIter  *b);
+void          gtk_tree_store_move             (GtkTreeStore *tree_store,
+                                               GtkTreeIter  *iter,
+                                               GtkTreePath  *position);
 
 
 #ifdef __cplusplus
